/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetLayout.cpp,v 2.11 2000/05/13 01:46:14 jeff Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qfile.h>
#include <qmsgbox.h>
#include <qobjcoll.h>
#include <qpainter.h>
#include <qstrlist.h>

#include "DialogWnd.h"
#include "GuiTools.h"

#include "GridDialog.h"
#include "DlgParser.h"
#include "ProjectWnd.h"
#include "SelectedWidgetBorder.h"
#include "WidgetModuleMgr.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"
#include "xml/XMLValue.h"

static const int MULTIPLE_COUNT = 4;   // # of grid spacing to move when ALT+ key combination is used

#define Inherited QObject


WidgetLayout::WidgetLayout
(
    DialogWnd*	parent,
    const char*	name
)
    :
    Inherited( (QObject *)parent, name ),
    itsParent( parent ),
    itsDlgEditWnd( NULL ),
    itsLayoutHelper( *this ),
    isDragging( FALSE),
    itsParser( NULL ),
    widgetInConstruction( NULL ),
    autoSelect( FALSE ),
    itsScreenFlags( SF_None ),	
    itsWindowMinSize( 0, 0 ),
    itsWindowMaxSize( QWIDGETSIZE_MAX, QWIDGETSIZE_MAX ),
    itsWindowPos( -1, -1 ),
    isTextTranslated( 
        ProjectWnd::GetAppConfig().readEntry( "TRDefault", 0L ) != 0 )
{
    itsSelectedWidget.setAutoDelete( TRUE );

    itsNoMoveWidgets.setAutoDelete( FALSE );
    itsDlgWidgets.setAutoDelete( TRUE );
    itsZOrdering.setAutoDelete( FALSE );

    itsWidgetPopup = new QPopupMenu;
    itsWidgetPopup->insertItem( tr( "Raise" ), 
                                this, 
                                SLOT(RaiseSelectedWidget()) );
    itsWidgetPopup->insertItem( tr("Lower" ), 
                                this, 
                                SLOT(LowerSelectedWidget()) );
    itsWidgetPopup->insertSeparator();
    itsWidgetPopup->insertItem( tr("Delete" ), 
                                this, 
                                SLOT(DeleteSelectedWidget()) );
    itsWidgetPopup->insertSeparator();
    itsWidgetPopup->insertItem( tr("Properties" ), 
                                this,	
				SLOT(SelectedWidgetProperties()) );

    itsWidgetFramePopup = new QPopupMenu;
    itsWidgetFramePopup->insertItem( tr("Properties" ), 
                                     this,
				     SLOT(SelectedWidgetProperties()) );
}


WidgetLayout::~WidgetLayout()
{
}


void WidgetLayout::AddWidget
(
    DlgWidget::Type	type
)
{
    DlgWidget* newWidget = DlgWidget::NewDlgWidget( itsDlgEditWnd, type );

    bool widgetOnTop = TRUE;

    if( type == DlgWidget::DW_ButtonGroup ||
	type == DlgWidget::DW_GroupBox )
    {
	widgetOnTop = FALSE;
    }

    AddWidget( newWidget, widgetOnTop );
}


void WidgetLayout::AddModuleWidget(
    int	theModuleNum,
    int	theModuleWidgetNum
    )
{
    DlgWidget* newWidget = WidgetModuleMgr::newWidget( theModuleNum,
						       theModuleWidgetNum,
						       itsDlgEditWnd );

    AddWidget( newWidget );
}


void WidgetLayout::AddModuleWidget(
    const QString&	theModuleName,
    const QString&	theModuleWidgetName
    )
{
    DlgWidget* newWidget = WidgetModuleMgr::newWidget( theModuleName,
						       theModuleWidgetName,
						       itsDlgEditWnd );
    
    AddWidget( newWidget );
}


void WidgetLayout::AddWidget(
    DlgWidget*	theNewWidget,
    bool	isWidgetOnTop
    )
{
    if( isWidgetOnTop )
    {
	itsZOrdering.append( theNewWidget );
    }
    else
    {
	itsZOrdering.insert( 0, theNewWidget );
    }
    
    QWidget* widget = theNewWidget->GetWidget();
    QString name = widget->name();

    itsDlgWidgets.insert( name, theNewWidget );

    ConnectToWidget( widget );    

    widget->show();
    SetDragging( widget );

    if( autoSelect )
	AddToSelection( widget );

    emit Changed();    
}
	

void WidgetLayout::ShowLayoutMgr( bool	show )
{
    itsLayoutHelper.Show( show );
}


void WidgetLayout::SetDlgEditWnd
(
    WidgetFrame*	editWnd
)
{
    itsDlgEditWnd = editWnd;
    itsLayoutHelper.SetLayoutWindow( editWnd );
}


void WidgetLayout::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "WidgetLayout" );

  generator.WriteStartBlock( "WidgetLayoutCommon" );
  generator.WriteXMLValue( "InitialPos", itsWindowPos );
  generator.WriteXMLValue( "Size", itsDlgEditWnd->size() );
  generator.WriteXMLValue( "MinSize", itsWindowMinSize );
  generator.WriteXMLValue( "MaxSize", itsWindowMaxSize );
  generator.WriteXMLValue( "GridSize", itsDlgEditWnd->GetGrid() );
  generator.WriteXMLValue( "TextTranslated", isTextTranslated );
  // End WidgetLayoutCommon
  generator.WriteEndBlock(); 

  generator.WriteStartList( "Widgets" );
  QListIterator< DlgWidget > widgetIter( itsZOrdering );
  DlgWidget* dlgWidget;
     
  while( (dlgWidget = widgetIter()) )
  {
    dlgWidget->Save( generator );
  }
  generator.WriteEndBlock();
  
  itsLayoutHelper.Save( generator );

  // End WidgetLayout
  generator.WriteEndBlock();
}


void WidgetLayout::ResetContents( void )
{
    //itsDlgWidgets.setAutoDelete( TRUE );
    itsDlgWidgets.clear();
    //itsDlgWidgets.setAutoDelete( FALSE );
    
    itsZOrdering.clear();
    itsNoMoveWidgets.clear();
}



void WidgetLayout::RestoreContents
(
    DlgParser *theParser
)
{
    // Remember the Parser 
    itsParser = theParser;
    
  // Connecting the Parser
    connect( itsParser, SIGNAL(NewWidget(QString &)), 
	     SLOT(RestoreNewWidget(QString &)));
    connect( itsParser, SIGNAL(WidgetKeyValue(QString &,QString &)), 
	     SLOT(RestoreWidgetKeyValue(QString &,QString &)) );
    connect( itsParser, SIGNAL(EndWidget()), 
	     SLOT(RestoreEndWidget()) );

    connect( itsParser, 
	     SIGNAL(LayoutKeyValue(QString &,QString &)),
	     SLOT(RestoreLayoutKeyValue(QString &,QString &)) );

    connect( itsParser, 
	     SIGNAL(EndLayout()), 
	     &itsLayoutHelper,
	     SLOT(ContentsRestored()) );
    connect( itsParser, 
	     SIGNAL(EndLayout()), 
	     SLOT(ContentsRestored()) );

    connect( itsParser, 
	     SIGNAL(NewLayoutItem(QString&)),
	     &itsLayoutHelper,
	     SLOT(RestoreNewLayoutItem(QString&)) );

    connect( itsParser, 
	     SIGNAL(LayoutItemKeyValue(QString&,QString&)),
	     &itsLayoutHelper,
	     SLOT(RestoreLayoutItemKeyValue(QString&,QString&)) );

    connect( itsParser, 
	     SIGNAL(EndLayoutItem()),
	     &itsLayoutHelper,
	     SLOT(RestoreEndLayoutItem()) );
}

void WidgetLayout::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "WidgetLayout" )
    return;

  const XMLValueMap map = value.AsNested();

  {
    const XMLValueMap common = map["WidgetLayoutCommon"].AsNested();

    itsDlgEditWnd->move( 0,0 );
    SetWindowPos( common["InitialPos"].AsQPoint() );
    SetWindowSize( itsInitialSize = common["Size"].AsQSize() );
    SetWindowMinSize( common["MinSize"].AsQSize() );
    SetWindowMaxSize( common["MaxSize"].AsQSize() );
    itsDlgEditWnd->SetGrid( common["GridSize"].AsInt() );
  }

  XMLValueList list = map["Widgets"].AsList();
  QListIterator<XMLValue> it( list );
  for( ; it.current(); ++it )
  {
    // Should be a DlgWidget...
    DlgWidget *widget = DlgWidget::NewDlgWidget( itsDlgEditWnd,
                                                 *(it.current()) );

    // ... but one could never know
    if( widget )
    {
      itsZOrdering.append( widget );

      itsDlgWidgets.insert( widget->GetWidget()->name(), 
                            widget );

      widget->GetWidget()->show();
        
      if( autoSelect )
        AddToSelection( widget->GetWidget() );

      ConnectToWidget( widget->GetWidget() );
    }
  }

  itsLayoutHelper.Restore( map["Layout"] );
  
}

/* Slot, called when all Widgets for this layout 
 * have been read
 *
 * Undo all connections done by RestoreContents
 */
void WidgetLayout::ContentsRestored()
{
    if( ! itsParser )
	return;

    itsParser->disconnect( this );
    itsParser->disconnect( &itsLayoutHelper );

    itsParser = NULL;
}

void WidgetLayout::RestoreNewWidget( QString &widgetName )
{
    // Create a new Widget

    // If the name contains :: then it's from a module, otherwise it's a
    // built in widget.
    int pos = widgetName.find( "::" );
    if( pos >= 0 )
    {
	QString moduleName = widgetName.left( pos );
	QString moduleWidgetName = 
	    widgetName.mid( pos + 2, widgetName.length() );

	widgetInConstruction = WidgetModuleMgr::newWidget( moduleName,
							   moduleWidgetName,
							   itsDlgEditWnd );
    }
    else
    {
	widgetInConstruction = DlgWidget::NewDlgWidget( itsDlgEditWnd, 
							widgetName );
    }
	    
    // No Widget => Error in File, stop action
    if( ! widgetInConstruction )
    {
	itsParser->StopParsing();
	return;
    }	
    else
    {
	ConnectToWidget( widgetInConstruction->GetWidget() );
    }
}

void WidgetLayout::RestoreLayoutKeyValue( QString &key, QString &value )
{

    // Still read in the depreciated Rect field
    if( key == "Rect" )
    {
	QTextStream read( value, IO_ReadOnly );
	QRect aRect;
	read >> aRect;
	itsDlgEditWnd->move( aRect.left(), aRect.top() );
	SetWindowSize( aRect.size() );
	itsWindowPos = QPoint( -1,-1 );
    } 
    else if( key == "InitialPos" )
    {
	QTextStream read( value, IO_ReadOnly );
	QPoint point;
	read >> point;
	SetWindowPos( point );
    }
    else if( key == "Size" )
    {
	QTextStream read( value, IO_ReadOnly );
	QSize size;
	read >> size;
	itsDlgEditWnd->move( 0,0 );
	SetWindowSize( size );
	itsInitialSize = size;
    }
    else if( key == "MinSize" )
    {
	QTextStream read( value, IO_ReadOnly );
	QSize size;
	read >> size;
	SetWindowMinSize( size );
    }
    else if( key == "MaxSize" )
    {
	QTextStream read( value, IO_ReadOnly );
	QSize size;
	read >> size;
	SetWindowMaxSize( size );
    }
    else if( key == "Grid" )
    {
	itsDlgEditWnd->SetGrid( value.toInt() );
    }
    else if( key == "TextTranslated" )
    {
        isTextTranslated = (value == "TRUE");
    }
    else
    {
        cerr << tr( "Unknown key value Pair: < %1 : %2 >" ).
            arg( key ).arg( value ) << endl;
    }
}

void WidgetLayout::RestoreWidgetKeyValue( QString &key, QString &value )
{
    // The Name token is of special interest for us
    if( key == "Name" && widgetInConstruction )
    {
	// Check for double names, and rename if necessary
	if( itsDlgWidgets[value] )
	{
	    itsDlgWidgets.insert( widgetInConstruction->GetWidget()->name(), 
				  widgetInConstruction );
	    return;
	} else
	{
	    itsDlgWidgets.insert( value, widgetInConstruction );
	}
    }

    // Just send the values to the widget
    if( ! widgetInConstruction->RestoreKeyValue( key, value ) )
    {
        cerr << tr( "Unknown key value Pair: < %1 : %2 >" ).
            arg( key ).arg( value ) << endl;
    }
}

void WidgetLayout::RestoreEndWidget( )
{
    if( widgetInConstruction )
    {
	widgetInConstruction->GetWidget()->show();
	itsZOrdering.append( widgetInConstruction );

	if( autoSelect )
	    AddToSelection( widgetInConstruction->GetWidget() );
    }
}


void WidgetLayout::GenerateDataHeader
(
 QTextStream&	stream,
 const QString&	dataName,
 const QString&	dataBaseName,
 const QString&	dataBaseHeader
 )
{
  stream << "#ifndef " << dataName << "_included\n"
         << "#define " << dataName << "_included\n"
         << endl;

  // find the list of all the header's we will need
  QListIterator< DlgWidget > iter( itsZOrdering );
  DlgWidget* widget;
  QDict<int> headerDict;
  QString header;
  int dummyInt = 0;

  // create a dictionary of header files that are needed for the mapped
  // variables, without duplicates
  while( (widget = iter()) )
  {
    header = widget->GetHeaderFilename();

    if( !headerDict[ header ] && widget->IsVariableMapped() )
    {
	    headerDict.insert( header, &dummyInt );
    }
  }


  QDictIterator<int> headerIter( headerDict );

  stream << "#include " << dataBaseHeader << '\n';

  while( headerIter.current() )
  {
    stream << "#include " << headerIter.currentKey() << '\n';
    ++headerIter;
  }
    
  stream << endl
         << "class " << dataName << " : public " << dataBaseName << '\n'
         << "{\n"
         << DlgWidget::srcIndent << "Q_OBJECT\n"
         << endl
         << "public:\n"
         << endl
         << DlgWidget::srcIndent << dataName << "(QWidget *parent = NULL, const char *name = NULL);\n"
         << endl
         << DlgWidget::srcIndent << "virtual ~" << dataName << "();\n";

  // define slots
    
  QDict<QString> publicSlots, protectedSlots;
      
  publicSlots.setAutoDelete( TRUE );
  protectedSlots.setAutoDelete( TRUE );

  GetAllSlots(&publicSlots, &protectedSlots, dataBaseName);
      
  QString* slotStr;

  if(publicSlots.count() > 0)
  {
    QDictIterator<QString> publicIter( publicSlots );

    stream << endl << DlgWidget::srcIndent << "public slots:\n\n";
    while( (slotStr = publicIter()) )
      stream << DlgWidget::srcIndent << "virtual void " << *slotStr << " =0;\n";
  }

  if(protectedSlots.count() > 0)
  {
    QDictIterator<QString> protectedIter( protectedSlots );
    
    stream << endl << DlgWidget::srcIndent << "protected slots:\n\n";
    while( (slotStr = protectedIter()) )
    {
      // if the user declared a slot to be both protected and private, make it public
      if( !publicSlots[ *slotStr ] )
        stream << DlgWidget::srcIndent << "virtual void " << *slotStr << " =0;\n";
    }
  }

  // declare mapped variables
    
  iter.toFirst();
  while( (widget = iter()) )
    if(widget->IsVariableMapped())
    {
        stream << endl <<  DlgWidget::srcIndent << "protected:\n\n";
        break;
    }

  iter.toFirst();
  while( (widget = iter()) )
  {
    if( widget->IsVariableMapped() )
    {
        stream << DlgWidget::srcIndent << widget->GetWidgetClass() << " *" <<
        widget->GetVariableName() << ";\n";
    }
  }
    
  stream << "};\n"
         << '\n'
         << "#endif // " << dataName << "_included\n";
}

void WidgetLayout::GenerateDataSource
(
    QTextStream&	stream,
    const QString&	dataHeader,
    const QString&	dataName,
    const QString&	dataBaseName,
    bool		isDialog,
    bool		isDialogModal,
    WFlags		windowFlags
)
{
    stream << "#include <qpixmap.h>\n"
           << "#include <qlayout.h>\n"
           << "#include \"" << dataHeader << "\"\n"
           << '\n';

    QDict<int> nonMappedClasses;
    QListIterator<DlgWidget> widgetIter( itsZOrdering );
    DlgWidget* widget;
    QString header;
    int dummyInt = 0;

    // create a dictionary of the header files needed for the widgets that
    // have not been mapped
    while( (widget = widgetIter()) )
    {
	header = widget->GetHeaderFilename();

	if( !nonMappedClasses[ header ] && !widget->IsVariableMapped() )
	{
	    nonMappedClasses.insert( header, &dummyInt );
	}
    }
    
    QDictIterator<int> headerIter( nonMappedClasses );
    
    while( headerIter.current() )
    {
	stream << "#include " << headerIter.currentKey() << '\n';
	++headerIter;
    }

    stream << '\n'
           << dataName << "::" << dataName << "(QWidget *parent, const char *name)\n";
    for(unsigned int i=0; i<strlen(dataName)+1; i++)
        stream << ' ';
    stream << ": " << dataBaseName << "(parent, name, ";
    
    if( isDialog )
	if( isDialogModal )
	    stream << "TRUE, ";
	else
	    stream << "FALSE, ";

    stream << windowFlags << ")\n"
           << "{\n";
    
    QDict<char> visitedChildren;
    GenerateSourceForChildren( itsDlgEditWnd, stream, visitedChildren );

    itsLayoutHelper.GenerateLayoutSource( stream );

    // Set initial position and size
    if( (itsWindowPos.x() >= 0) && (itsWindowPos.y() >= 0) )
	stream << DlgWidget::srcIndent 
               << "setGeometry(" << itsWindowPos.x() << "," 
               << itsWindowPos.y() << ", " 
               << itsDlgEditWnd->width() << "," 
               << itsDlgEditWnd->height() << ");\n";
    else
	stream << DlgWidget::srcIndent 
               << "resize(" << itsDlgEditWnd->width() << "," 
               << itsDlgEditWnd->height() <<  ");\n";
    
    // Set min and max sizes
    stream << DlgWidget::srcIndent 
           << "setMinimumSize(" <<  itsWindowMinSize.width() << ", " 
           << itsWindowMinSize.height() <<  ");\n"
           << DlgWidget::srcIndent 
           << "setMaximumSize(" << itsWindowMaxSize.width()
           << ", " << itsWindowMaxSize.height() << ");\n"
           << "}\n"
           << "\n";

    // define the destructor

    stream << dataName << "::~" << dataName << "()\n"
           << "{\n"
           << "}\n";
}


void WidgetLayout::GenerateSourceForChildren(
    QWidget*            parent,
    QTextStream&        stream,
    QDict<char>&        visitedChildren
    )
{
    const char* parentName = 0;
    if( parent )
    {
        parentName = parent->name();
    }
    
    if( !parentName || visitedChildren[ parentName ] )
    {
        return;
    }

    visitedChildren.insert( parentName, parentName );

    const QObjectList* childList = parent->children();
    if( childList )
    {
        const QObject* child = 0;
        QListIterator<QObject> childIter( *childList );
        while( (child = childIter()) )
        {
            const char* name = child->name();
            if( name )
            {
                DlgWidget* widget = itsDlgWidgets[ name ];
                if( widget && !visitedChildren[ name ] )
                {
                    GenerateWidgetSource( widget, stream );

                    GenerateSourceForChildren( widget->GetWidget(), 
                                               stream,
                                               visitedChildren );
                }
            }
        }
    }
}
                

void WidgetLayout::GenerateWidgetSource
(
    DlgWidget*		widget,
    QTextStream&	stream
)
{
    QString varName = widget->GetGeneratedVarName();
    
    stream << DlgWidget::srcIndent;

    // construct a new widget
    if( !widget->IsVariableMapped() )
    {
	// declare new variable
	stream << widget->GetWidgetClass() << " *";
    }

    stream << varName << " = new " << widget->GetWidgetClass() << '(';

    // widget's parent for source may have bben set from a pervious code
    // generation.
    widget->ClearParentForSource();

    // Determine the parent for the widget
    QString parent = "this";
    QWidget* parentWidget = widget->GetWidget()->parentWidget();
    if( parentWidget != itsDlgEditWnd ) 
    {
	parent = GetDlgWidget( parentWidget )->GetGeneratedVarName();
    }
    else
    {
        DlgWidget *dlgParent;

        if((dlgParent=widget->GetParentForCodeGeneration(itsZOrdering))!=NULL)
            parent=dlgParent->GetGeneratedVarName();
    }
 
    // 'new' variable
    widget->GenerateCtor(stream, parent);
    stream << ");\n";

    varName = DlgWidget::srcIndent + varName + widget->Dot();
    widget->GenerateSource( stream, varName, *this );
    widget->GenerateSourceConnects(stream, varName); 
    stream << '\n';
}


void WidgetLayout::SetDragging
(
    QWidget*	widget
)
{
    ClearSelection();
    AddToSelection( widget );
    WidgetGeometryChange( widget );
    
    UpdateCursor( widget );
    
    itsWidgetClick.setX( widget->width()/2 );
    itsWidgetClick.setY( widget->height()/2 );
    
    widget->move( 0,0 );
}


void WidgetLayout::ConnectToWidget( QWidget* widget )
{
    // Install event filters into the widget's children in case they
    // stop the mouse events.
    QObjectList* childList = widget->queryList();
    QObjectListIt iter( *childList );
    QObject* child;
    while( (child = iter()) )
    {
	child->installEventFilter( this );
    }
    delete childList;

    widget->installEventFilter( this );

    DlgWidget* dlgWidget = GetDlgWidget( widget );
    if( dlgWidget )
    {
	connect( dlgWidget, 
		 SIGNAL(LayoutStatusChanged(const DlgWidget*)),
		 SLOT(UpdateWidgetLayoutState(const DlgWidget*)) );
    }
}


void WidgetLayout::AlignTop()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected *sel = iter();
	int top = sel->widget->geometry().top();
	QWidget* widget;
    
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;
	    
	    // make sure you don't mave one that isn't supposed to be moved
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		widget->move( widget->x(), top );	
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::AlignBottom()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected *sel = iter();
	int bottom = sel->widget->geometry().bottom();
	QRect rect;
	QWidget* widget;
    
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;
	    
	    // make sure you don't mave one that isn't supposed to be moved
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		rect = widget->geometry();
		rect.moveBottomRight( QPoint( rect.right(), bottom ) );
		widget->setGeometry( rect );
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::AlignLeft()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected *sel = iter();
	int left = sel->widget->geometry().left();
	QWidget* widget;
    
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;
	    
	    // make sure you don't mave one that isn't supposed to be moved
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		widget->move( left, widget->y() );
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::AlignRight()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected* sel = iter();
	int right = sel->widget->geometry().right();
	QRect rect;
	QWidget* widget;
    
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;
	    
	    // make sure you don't mave one that isn't supposed to be moved
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		rect = widget->geometry();
		rect.moveBottomRight( QPoint( right, rect.bottom() ) );
		widget->setGeometry( rect );
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::SizeHorizontal()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected* sel = iter();
	int width = sel->widget->width();
	QWidget* widget;
	
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;

	    // make sure you don't size one that isn't supposed to be sized
	    // no move == no size
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		widget->resize( width, widget->height() );
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::SizeVertical()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );

	Selected* sel = iter();
	int height = sel->widget->height();
	QWidget* widget;
	
	// skip first item
	while( (sel = iter()) )
	{
	    widget = sel->widget;

	    // make sure you don't size one that isn't supposed to be sized
	    // no move == no size
	    if( itsNoMoveWidgets[widget->name()] == NULL )
	    {
		widget->resize( widget->width(), height );
		emit Changed();
	    }
	}
    }
}


void WidgetLayout::SizeBothHV()
{
    SizeHorizontal();
    SizeVertical();
}


void WidgetLayout::SizeToFit()
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;
	
	while( (sel = iter()) )
	{
	    QSize hint = sel->widget->sizeHint();
	    if( !hint.isValid() )
	    {
		// if no size hint given, don't change size
		hint = sel->widget->size();
	    }

	    ResizeWidget( sel->widget, QRect(sel->widget->pos(), hint) );
	}
    }
}


void WidgetLayout::MoveUp(int multiple)
{
    MoveWidget( QPoint(0, -itsDlgEditWnd->GetGrid())*multiple );
}


void WidgetLayout::MoveDown(int multiple)
{
    MoveWidget( QPoint(0, itsDlgEditWnd->GetGrid())*multiple );
}


void WidgetLayout::MoveLeft(int multiple)
{
    MoveWidget( QPoint(-itsDlgEditWnd->GetGrid()*multiple, 0) );
}


void WidgetLayout::MoveRight(int multiple)
{
    MoveWidget( QPoint(itsDlgEditWnd->GetGrid()*multiple, 0) );
}


void WidgetLayout::MoveWidget( const QPoint&	newPos )
{
    if( !itsSelectedWidget.isEmpty() )
    {
	// send a fake mouse move event

	QWidget* widget = itsSelectedWidget.first()->widget;
	QMouseEvent event( QEvent::MouseMove, newPos, LeftButton, LeftButton );

	// remember old state of dragging and point of widget click and cursor
	bool oldDragging = isDragging;
	isDragging = TRUE;

	QPoint oldClick = itsWidgetClick;
	itsWidgetClick = QPoint(0,0);

	ProcessMouseMove( widget, &event );

	itsWidgetClick = oldClick;
	isDragging = oldDragging;
    }
}


void WidgetLayout::ResizeShorter(int multiple)
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;
	
	while( (sel = iter()) )
	{
	    QRect widgetRect = sel->widget->geometry();
	    widgetRect.setBottom( widgetRect.bottom() - 
				  (itsDlgEditWnd->GetGrid()*multiple) );
	    
	    ResizeWidget( sel->widget, widgetRect );
	}
    }
}


void WidgetLayout::ResizeWider(int multiple)
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;
	
	while( (sel = iter()) )
	{
	    QWidget* parent = sel->widget->parentWidget();
	    QRect widgetRect = sel->widget->geometry();
	    widgetRect.setRight( widgetRect.right() + 
				 (itsDlgEditWnd->GetGrid()*multiple) );

	    // make sure the widgetRect doesn't go off the widget frame
	    widgetRect.setRight( QMIN( widgetRect.right(), 
				       parent->width() - 1 ) );
	    
	    ResizeWidget( sel->widget, widgetRect );
	}
    }
}


void WidgetLayout::ResizeNarrower(int multiple)
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;
	
	while( (sel = iter()) )
	{
	    QRect widgetRect = sel->widget->geometry();
	    widgetRect.setRight( widgetRect.right() - 
				 (itsDlgEditWnd->GetGrid()*multiple) );
	    
	    ResizeWidget( sel->widget, widgetRect );
	}
    }
}


void WidgetLayout::ResizeTaller(int multiple)
{
    if( !itsSelectedWidget.isEmpty() )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;
	
	while( (sel = iter()) )
	{
	    QWidget* parent = sel->widget->parentWidget();
	    QRect widgetRect = sel->widget->geometry();
	    widgetRect.setBottom( widgetRect.bottom() + 
				  (itsDlgEditWnd->GetGrid()*multiple) );

	    // make sure the widgetRect doesn't go off the widget frame
	    widgetRect.setBottom( QMIN( widgetRect.bottom(), 
					parent->height() - 1 ) );
	    
	    ResizeWidget( sel->widget, widgetRect );
	}
    }
}

void WidgetLayout::MoveUp1()
{
    MoveUp(1);
}
 
void WidgetLayout::MoveDown1()
{
    MoveDown(1);
}
 
void WidgetLayout::MoveLeft1()
{
    MoveLeft(1);
}

void WidgetLayout::MoveRight1()
{
    MoveRight(1);
}
 
void WidgetLayout::ResizeShorter1()
{
    ResizeShorter(1);
}
 
void WidgetLayout::ResizeWider1()
{
    ResizeWider(1);
}
 
void WidgetLayout::ResizeNarrower1()
{
    ResizeNarrower(1);
}
 
void WidgetLayout::ResizeTaller1()
{
    ResizeTaller(1);
}

void WidgetLayout::MoveUpMultiple()
{
    MoveUp(MULTIPLE_COUNT);
}
 
void WidgetLayout::MoveDownMultiple()
{
    MoveDown(MULTIPLE_COUNT);
}
 
void WidgetLayout::MoveLeftMultiple()
{
    MoveLeft(MULTIPLE_COUNT);
}

void WidgetLayout::MoveRightMultiple()
{
    MoveRight(MULTIPLE_COUNT);
}
 
void WidgetLayout::ResizeShorterMultiple()
{
    ResizeShorter(MULTIPLE_COUNT);
}
 
void WidgetLayout::ResizeWiderMultiple()
{
    ResizeWider(MULTIPLE_COUNT);
}
 
void WidgetLayout::ResizeNarrowerMultiple()
{
    ResizeNarrower(MULTIPLE_COUNT);
}
 
void WidgetLayout::ResizeTallerMultiple()
{
    ResizeTaller(MULTIPLE_COUNT);
}
    
void WidgetLayout::ResizeWidget( QWidget*	widget,
				 const QRect&	newRect )
{
    widget->move( newRect.left(), newRect.top() );

    // currently only the widget frame is not moveable
    if( itsNoMoveWidgets[ widget->name() ] != NULL )
    {
	QRect origRect = widget->geometry();
	QSize min = widget->minimumSize();
	QSize max = widget->maximumSize();
		
	// Only set the min/max sizes for the dimensions that changed
	if( origRect.width() != newRect.width() )
	{
	    widget->setMinimumSize( newRect.width(), min.height() );
	    widget->setMaximumSize( newRect.width(), max.height() );
	}
		
	if( origRect.height() != newRect.height() )
	{
	    widget->setMinimumSize( min.width(), newRect.height() );
	    widget->setMaximumSize( max.width(), newRect.height() );
	}
    }
    else
    {
	widget->resize( newRect.width(), newRect.height() );
    }
	    
	    
    // WidgetGeometryChange now called in ProcessResize since the 
    // widget was just resized in the previous code
    //WidgetGeometryChange( widget );
    emit Changed();
}


bool WidgetLayout::eventFilter(
    QObject*	theSender,
    QEvent*	theEvent
    )
{
    // Default to passing the event on.
    bool rc = FALSE;
    
    QWidget* widget = 0;

    if( (theSender == itsDlgEditWnd) ||
	(theSender->parent() == itsDlgEditWnd) )
    {
	// The sender is one of the widgets we manage.
	widget = static_cast<QWidget*> (theSender);
    }
    else
    {
	// Trace up the parent hierarchy to find the widget we manage.
	QObject* obj = theSender;
	while( obj && (!obj->name() || !GetWidget( obj->name() )) )
	{
	    obj = obj->parent();
	}
	
	widget = static_cast<QWidget*> (obj);
    }

    if( widget == 0 )
    {
	return rc;
    }

    switch( theEvent->type() )
    {
	case QEvent::MouseButtonPress:
	{
	    QMouseEvent* mouseEvent = static_cast<QMouseEvent*> (theEvent);

	    ProcessMousePress( widget, mouseEvent );

	    // Stop the event.
	    rc = TRUE;

	    break;
	}
	case QEvent::MouseButtonRelease:
	{
	    isDragging = FALSE;

	    // Stop the event.
	    rc = TRUE;

	    break;
	}
	case QEvent::MouseButtonDblClick:
	{
	    QMouseEvent* mouseEvent = static_cast<QMouseEvent*> (theEvent);

	    if( mouseEvent->button() == LeftButton )
	    {
		SelectedWidgetProperties();
	    }

	    // Stop the event.
	    rc = TRUE;

	    break;
	}
	case QEvent::MouseMove:
	{
	    QMouseEvent* mouseEvent = static_cast<QMouseEvent*> (theEvent);

	    ProcessMouseMove( widget, mouseEvent );
	    
	    // Stop the event.
	    rc = TRUE;

	    break;
	}
	case QEvent::Move:
	case QEvent::Resize:
	{
	    WidgetGeometryChange( widget );

	    break;
	}
	default:
	{
	    break;
	}
    }

    return rc;
}


void WidgetLayout::ProcessMousePress
(
    QWidget* 		widget,
    QMouseEvent*	event
)
{
    switch( event->button() )
    {
	case LeftButton:
	{
	    if( (event->state() & ControlButton) && 
		(widget != itsDlgEditWnd ) )
	    {
		if( IsSelected( widget ) )
		{
		    RemoveFromSelection( widget );
		}
		else
		{
		    AddToSelection( widget );
		}
	    }
	    else if( !IsSelected( widget ) || (itsSelectedWidget.count() > 1) )
	    {
		ClearSelection();
		AddToSelection( widget );
	    }

	    UpdateCursor( widget );
	
	    itsWidgetClick = event->pos();
	    isDragging = TRUE;	
	
	    break;
	}
	
	case MidButton:
	{
	    ClearSelection();
	    emit WidgetDeselected();
	    break;
	}
		
	case RightButton:
	{
	    ClearSelection();
	    AddToSelection( widget );

	    DlgWidget* dlgWidget = GetDlgWidget( widget );
	    if( dlgWidget )
	    { 
		itsWidgetPopup->popup( QCursor::pos() );
	    }
	    else if( widget == itsDlgEditWnd )
	    {
		itsWidgetFramePopup->popup( QCursor::pos() );
	    }
	    
	    break;
	}	
	case NoButton:
	case MouseButtonMask:
	case ShiftButton:
	case ControlButton:
	case AltButton:
	case KeyButtonMask:
	{
	    break;
	}
    }
}


void WidgetLayout::ProcessMouseMove
(
    QWidget*	 	widget,
    QMouseEvent*	event
)
{
    if( isDragging && 
	itsSelectedWidget.count() > 0 )              // Moving only
    {
	QPoint oldWidgetPos = widget->pos();
	QRect  newWidgetRect = CalculateGeometryRect( widget, event->pos() );
	QPoint newWidgetPos = newWidgetRect.topLeft();
	bool   moveOKForAll = TRUE;
	int    maxWidth  = itsDlgEditWnd->width();
	int    maxHeight = itsDlgEditWnd->height();

	if( oldWidgetPos != newWidgetPos )
	{
	    QPoint delta = newWidgetPos - oldWidgetPos;
	    QListIterator<Selected> iter( itsSelectedWidget );
	    Selected *sel;

	    while( (sel = iter()) && moveOKForAll )
	    {
		// don't move one that's not allowed
		if( itsNoMoveWidgets[ sel->widget->name() ] )
		{
		    moveOKForAll = FALSE;
		}
		else
		{
		    if( sel->widget == widget )
		    {
			sel->newPos = newWidgetPos;
		    }
		    else
		    {
			sel->newPos = sel->widget->pos() + delta;
			int x = sel->newPos.x();
			int y = sel->newPos.y();

			if( x < 0 || x+sel->widget->width() > maxWidth ||
			    y < 0 || y+sel->widget->height() > maxHeight )
			{
			    moveOKForAll = FALSE;
			}
		    }
		}
	    }

	    if( moveOKForAll )
	    {
		iter.toFirst();

		while( (sel = iter()) )
		{
		    sel->widget->move( sel->newPos );
		}
	  
		WidgetGeometryChange( widget );
		emit Changed();
	    }      
	}
    } 

    UpdateCursor( widget );
}


void WidgetLayout::UpdateWidgetLayoutState( const DlgWidget*    theDlgWidget )
{
    QWidget* widget = theDlgWidget->GetWidget();

    if( theDlgWidget->IsInLayout() )
    {
        itsNoMoveWidgets.insert( widget->name(), widget );
    }
    else
    {
        itsNoMoveWidgets.remove( widget->name() );
    }

    Selected selectedWidget( widget );
    if( itsSelectedWidget.find( &selectedWidget ) != -1 )
    {
	Selected* sel = itsSelectedWidget.current();
	
	if( sel->myWidgetBorder )
	{
	    sel->myWidgetBorder->setMoveable( !theDlgWidget->IsInLayout() );
	}
    }

    WidgetGeometryChange( widget );
}


void WidgetLayout::RaiseSelectedWidget()
{
    QListIterator<Selected> iter( itsSelectedWidget );
    Selected* sel;
    
    while( (sel = iter()) )
    {
        sel->widget->raise();

	DlgWidget* dlgWidget = GetDlgWidget( sel->widget );
	if( itsZOrdering.removeRef( dlgWidget ) )
	{
	    itsZOrdering.append( dlgWidget );
	}
    
	emit Changed();
    }
}


void WidgetLayout::LowerSelectedWidget()
{
    QListIterator<Selected> iter( itsSelectedWidget );
    Selected* sel;
    
    while( (sel = iter()) )
    {
	sel->widget->lower();

	DlgWidget* dlgWidget = GetDlgWidget( sel->widget );
	if( itsZOrdering.removeRef( dlgWidget ) )
	{
	    itsZOrdering.insert( 0, dlgWidget );
	}
	emit Changed();
    }
}


void WidgetLayout::DeleteSelectedWidget()
{
    // deleting a widget
    if( itsSelectedWidget.count() > 0 )
    {
	QListIterator<Selected> iter( itsSelectedWidget );
	Selected* sel;

	while( (sel = iter()) )
	{
	    DlgWidget* dlgWidget = GetDlgWidget( sel->widget );
	    if( dlgWidget )
	    {
		itsZOrdering.removeRef( dlgWidget );
	
		QString name = sel->widget->name();

		if( dlgWidget->IsInLayout() )
		{
		    itsLayoutHelper.DeleteWidget( dlgWidget );
		}

		// remove will delete dlgWidget but not the actual widget
		itsDlgWidgets.remove( name ); 
		delete sel->widget;
	
		itsNoMoveWidgets.remove( name );

		emit Changed();
	    }
	}
	itsSelectedWidget.clear();
    }
    
    RemoveScreenFlags( SF_WidgetSelected | SF_Align );
}

void WidgetLayout::SelectedWidgetProperties()
{
    if( itsSelectedWidget.count() > 1 )
	return;

    Selected *sel = itsSelectedWidget.first();
    DlgWidget* dlgWidget;

    if( sel && (dlgWidget = GetDlgWidget(sel->widget)) )
    {
	// Remember the name first
	QString origName( sel->widget->name() );

	if( dlgWidget->DoPropertyDialog() == QDialog::Accepted )
	{
	    emit Changed();

	    // Check the name and register if neccesary
	    QString newName( sel->widget->name() );

	    // user might have changed the widget's name or variable mapping
	    emit WidgetSelected( newName, dlgWidget->GetVariableName() );

	    if( newName != origName )
	    {
		if( itsDlgWidgets[newName] )
		{
		    QMessageBox::message( 
                        tr( "Name conflict..." ),
                        tr( "The new widget name is already in use\nThe old name is restored !" ),
                        tr( "OK" ) );
                    sel->widget->setName( origName );
		} else
		{
		    dlgWidget = itsDlgWidgets.take( origName );
		    itsDlgWidgets.insert( newName, dlgWidget );
		}
	    }
	}
    }
    // Do properties on the whole window
    else
    {
	if( itsParent->DoPropertyDialog() == QDialog::Accepted )
	{
	    emit Changed();
	}
    }
}

void WidgetLayout::OptionsGrid( )
{
    GridDialog gridDialog( NULL, "GridDialog", itsDlgEditWnd->GetGrid() );
    if( gridDialog.exec() == QDialog::Accepted )
    {
	itsDlgEditWnd->SetGrid( gridDialog.GetNewGrid() );
	emit Changed();
    }
}

void WidgetLayout::EnableLayout()
{

    if( ! itsDlgEditWnd )
	return;

    itsDlgEditWnd->installEventFilter( this );
}
 

void WidgetLayout::SetSelected
(
    QWidget*	widget,
    bool	selected
)
{
    if( widget )
    {
	widget->setMouseTracking( selected );
    
	widget->update();

	if( selected )
	{
	    WidgetGeometryChange( widget );

	    QString varName;
	    DlgWidget* dlgWidget = itsDlgWidgets[ widget->name() ];
	    if( dlgWidget )
	    {
		varName = dlgWidget->GetVariableName();
	    }
	    
	    emit WidgetSelected( widget->name(), varName );
	}
    }
}

void WidgetLayout::ClearSelection
(
)
{
    QListIterator<Selected> iter( itsSelectedWidget );
    Selected* sel;
    
    while( (sel = iter()) )
    {
	SetSelected( sel->widget, FALSE );
	sel->widget->setCursor( arrowCursor );
    }

    itsSelectedWidget.clear();

    RemoveScreenFlags( SF_WidgetSelected | SF_Align );
}

void WidgetLayout::AddToSelection
(
    QWidget *widget
)
{
    if( ! IsSelected( widget ) )
    {
	DlgWidget* dlgWidget = GetDlgWidget( widget );
	if( dlgWidget )
	{
	    itsSelectedWidget.append( new Selected( *dlgWidget, 
						    itsDlgEditWnd ) );
	}
	else
	{
	    itsSelectedWidget.append( new Selected( widget, 
						    itsDlgEditWnd,
						    FALSE /*isMoveable*/ ) );
	}
    
	SetSelected( widget, TRUE );
	WidgetGeometryChange( widget );
    }

    if( itsSelectedWidget.count() == 1 )
    {
	AddScreenFlags( SF_WidgetSelected );
	RemoveScreenFlags( SF_Align );
    }
    else
    {
	AddScreenFlags( SF_WidgetSelected | SF_Align );
    }
}

void WidgetLayout::RemoveFromSelection
(
    QWidget *widget
)
{
    QListIterator<Selected> iter( itsSelectedWidget );
    Selected *sel;

    while( (sel = iter()) && (sel->widget != widget) );

    if( sel )
    {
	// SetSelected( sel->widget, FALSE );
	//sel->widget->setCursor( arrowCursor );
	itsSelectedWidget.remove( sel );
	SetSelected( widget, FALSE );
	WidgetGeometryChange( widget );
    }

    if( itsSelectedWidget.count() == 1 )
    {
	AddScreenFlags( SF_WidgetSelected );
	RemoveScreenFlags( SF_Align );
    }
    else if( itsSelectedWidget.count() > 0 )
    {
	AddScreenFlags( SF_WidgetSelected | SF_Align );
    }
}


bool WidgetLayout::IsSelected( QWidget*	widget )
{
    Selected sel( widget );
    return (itsSelectedWidget.find( &sel ) >= 0);
}


void WidgetLayout::UpdateCursor
(
    QWidget*	widget
)
{
    if( !isDragging )
    {
	if( IsSelected( widget ) )
	{
	    widget->setCursor( sizeAllCursor );
	}
	else
	{
	    widget->setCursor( arrowCursor );
	}
    }
}


void WidgetLayout::UpdateNoMoveWidgets( void )
{
    QDictIterator<DlgWidget> widgetIter( itsDlgWidgets );
    DlgWidget* dlgWidget;
    while( (dlgWidget = widgetIter()) )
    {
	QWidget* widget = dlgWidget->GetWidget();
	bool changed = FALSE;

	if( dlgWidget->IsInLayout() )
	{
	    // Make sure the widget is in the no move list
	    if( !itsNoMoveWidgets[ widget->name() ] )
	    {
		itsNoMoveWidgets.insert( widget->name(), widget );
		changed = TRUE;
	    }
	}
	else
	{
	    // Make sure the widget is not in the no move list
	    if( itsNoMoveWidgets[ widget->name() ] )
	    {
		itsNoMoveWidgets.remove( widget->name() );
		changed = TRUE;
	    }
	}

	if( changed )
	{
	    widget->repaint();
	    WidgetGeometryChange( widget );
	}
    }
}


void WidgetLayout::WidgetGeometryChange( QWidget* widget )
{
    if( widget == itsDlgEditWnd )
    {
	emit WidgetGeometryChanged( 0, 0, 
				    widget->width(), widget->height() );
    }
    else
    {
	DlgWidget* dlgWidget = GetDlgWidget( widget );
	if( dlgWidget )
	{
	    QPoint pos = dlgWidget->GetPos();
	
	    emit WidgetGeometryChanged( pos.x(), pos.y(),
					widget->width(), widget->height() );
	}
    }
}


QRect WidgetLayout::CalculateGeometryRect
(
    QWidget*	widget,
    QPoint	pos
)
{
    QPoint eventPos = widget->mapToParent( pos );
    QRect rect;
    QRect widgetRect = widget->geometry();
    int grid = itsDlgEditWnd->GetGrid();

    rect = widgetRect;
    QPoint gridPoint = eventPos - itsWidgetClick;
    if( grid > 1 )
    {
	gridPoint /= grid;
	gridPoint *= grid;
    }
    rect.moveTopLeft( gridPoint );

    QRect parentRect = widget->parentWidget()->geometry();

    if( rect.left() < 0 )
    {
	rect.moveTopLeft( QPoint(0,rect.top()) );
    }
	    
    if( rect.top() < 0 )
    {
	rect.moveTopLeft( QPoint(rect.left(),0) );
    }

    if( rect.right() > parentRect.width() - 1 )
    {
	rect.moveBottomRight(QPoint(parentRect.width() - 1,rect.bottom()));
    }
	    
    if( rect.bottom() > parentRect.height() - 1 )
    {
	rect.moveBottomRight(QPoint(rect.right(),parentRect.height() - 1));
    }

    return rect;
}


bool WidgetLayout::CopySelected( QtArchGenerator &generator )
{
  if( itsSelectedWidget.isEmpty() )
    return FALSE;

  generator.WriteStartBlock( "WidgetLayout" );

  QListIterator<Selected> iter( itsSelectedWidget );
  Selected* sel;

  while( (sel = iter()) )
  {
    DlgWidget *dlgWidget = GetDlgWidget( sel->widget );
    if( ! dlgWidget )
	    continue;

    dlgWidget->Save( generator );
  }

  generator.WriteEndBlock();

  return TRUE;
}


bool WidgetLayout::CutSelected( QtArchGenerator &generator )
{
    if( CopySelected( generator ) )
    {
	DeleteSelectedWidget();
	return TRUE;
    }

    return FALSE;
}


DlgWidget* WidgetLayout::GetDlgWidget
(
    const QWidget*	widget 
)
{
    DlgWidget* dlgWidget = NULL;

    if( widget )
    {
	QString name( widget->name() );
	dlgWidget = itsDlgWidgets[ name ];	
    }

    return dlgWidget;
}


void WidgetLayout::GetWidgetNames
( 
    QStrList&	nameList
) const
{
    QDictIterator<DlgWidget> iter( itsDlgWidgets );

    nameList.clear();
    while( iter.current() )
    {
	nameList.inSort( iter.currentKey() );
	++iter;
    }
}


void WidgetLayout::GetWidgetTypeNames( DlgWidget::Type	type,
				       QStrList&	nameList ) const
{
    QDictIterator<DlgWidget> iter( itsDlgWidgets );

    nameList.clear();
    while( iter.current() )
    {
	if( iter.current()->GetType() == type )
	{
	    nameList.inSort( iter.currentKey() );
	}
	
	++iter;
    }
}

    
void WidgetLayout::GetLayoutWidgetNames( QStrList&	theNameList,
					 bool		theInLayoutFlag ) const
{
    QDictIterator<DlgWidget> iter( itsDlgWidgets );
    
    theNameList.clear();
    while( iter.current() )
    {
	if( iter.current()->IsInLayout() == theInLayoutFlag )
	{
	    theNameList.inSort( iter.currentKey() );
	}
	
	++iter;
    }
}

void WidgetLayout::GetAllSlots(QDict<QString> *publicSlots, QDict<QString> *protectedSlots, const QString &dataBaseName)
{
  QListIterator< DlgWidget > iter(itsZOrdering);
  DlgWidget* widget;
  const QStrList& qdialogSlots = GuiTools::GetQDialogSlots();
  QString slot;

  SignalConnection* sig;
  // scan through all the widgets and their slots, placing the slots into
  // the correct dictionary for the scope making sure there are no 
  // duplicates
  iter.toFirst();
  while( (widget = iter()) )
  {
    QListIterator<SignalConnection> 
      signalIter( widget->GetConnectedSignalList() );
	
    while( (sig = signalIter()) )
    {
      // Signals which connect to base class slots must not be generated
      if( sig->GetScope() == SignalConnection::BaseClass )
        continue;

      slot = sig->GetSlot();

      // if the base is a dialog and the slot is defined by the QDialog
      // class, skip it
      if( dataBaseName != "QDialog" || !qdialogSlots.contains( slot ) )
      {
        switch( sig->GetScope() )
        {
        case SignalConnection::Protected:
          {
            if( !(*protectedSlots)[ slot ] )
            {
              protectedSlots->insert( slot, new QString( slot ) );
            }
            break;
          }
        case SignalConnection::Public:
          {
            if( !(*publicSlots)[ slot ] )
            {
              publicSlots->insert( slot, new QString( slot ) );
            }
            break;
          }
        case SignalConnection::BaseClass:
          break;
        }
      }
    }
  }
}

WidgetLayout::Selected::Selected(
    QWidget*		theWidget,
    WidgetFrame*	theWidgetFrame,
    bool		isMoveable
    )
    :
    widget( theWidget ),
    myWidgetBorder( 0 )
{
    if( theWidgetFrame )
    {
	myWidgetBorder = new SelectedWidgetBorder( theWidget,
						   theWidgetFrame,
						   isMoveable );
    }
}


WidgetLayout::Selected::Selected(
    DlgWidget&		theWidget,
    WidgetFrame*	theWidgetFrame
    ) :
    widget( theWidget.GetWidget() ),
    myWidgetBorder( new SelectedWidgetBorder( theWidget.GetWidget(), 
					      theWidgetFrame,
					      !theWidget.IsInLayout() ) )
{
}


WidgetLayout::Selected::~Selected()
{
    delete myWidgetBorder;
}
