/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetLayout.h,v 2.6 2000/04/29 20:15:45 klaus Exp $

*******************************************************************************/


#ifndef WidgetLayout_included
#define WidgetLayout_included

#include <qobject.h>
#include <qwidget.h>
#include <qlist.h>
#include <qdict.h>
#include <qrect.h>
#include <qcursor.h>
#include <qpopmenu.h>
#include <qstrlist.h>

#include "WidgetFrame.h"

#include "layout/LayoutHelper.h"
#include "widget/DlgWidget.h"

class QtArchGenerator;
class XMLValue;
class DlgParser;
class DialogWnd;
class SelectedWidgetBorder;

class WidgetLayout : public QObject
{
  Q_OBJECT

    friend class LayoutHelper;
    
  public: // methods
    
  WidgetLayout(
    DialogWnd*	parent = NULL,
    const char*	name = NULL
    );


  virtual ~WidgetLayout();
    
  void Start( void );
  // Start the widget layout

  void EnableLayout();
  // Enable interaction

  void AddWidget( 	
    DlgWidget::Type	type
    );

  void AddModuleWidget(
    int	theModuleNum,
    int	theModuleWidgetNum
    );

  void AddModuleWidget(
    const QString&	theModuleName,
    const QString&	theModuleWidgetName
    );

  void ShowLayoutMgr( bool	show );
  // Show or hide the layout manager window

  void SetCaption( 
    const QString&	theCaption
    );
  // Set the caption for the layout
    
  void SetNoMove(
    QWidget*	widget
    );
  // don't allow user to move top left corner

  void GetWidgetNames(
    QStrList&	nameList
    ) const;
  // Get a list of the names of the widgets

  void GetWidgetTypeNames(
    DlgWidget::Type type,
    QStrList&	nameList
    ) const;
  // Get a list of the names of the given type of widget

  void GetLayoutWidgetNames( 
    QStrList&	theNameList,
    bool		theInLayoutFlag = TRUE ) const;
  // Get a list of the names of the widgets which are in or are not in
  // a layout

  const DlgWidget* GetWidget(	
    const QString&	name
    ) const;
  // Given a name of a widget, return it or NULL if none match

  void SetDlgEditWnd(
    WidgetFrame*	editWnd
    );
  // set the window used to hold the widgets

  WidgetFrame *GetDlgEditWnd() const;
  // retrieve the window used to hold the widgets

  void ClearSelection();
  // Clear the widget selection

  void AddToSelection( const DlgWidget* widget );
  // Add the given widget to the selected widgets

  bool IsSelected( const DlgWidget* widget );
  // Is the given widget selected


  //
  // Saving and loading
  //
  void Save( QtArchGenerator & );
  // save contents, including the widgets

  void ResetContents( void );
  // reset the contents of the layout

  void RestoreContents(
    DlgParser *
    );
  void RestoreXML( const XMLValue & );
  // restore contents, creating the widgets


  //
  // Code generation
  //
  void GenerateDataHeader(
    QTextStream&	stream,
    const QString&	dataName,
    const QString&	dataBaseName,
    const QString&	dataBaseHeader
    );
  // generate the dialog data header file

  void GenerateDataSource(
    QTextStream&	stream,
    const QString&	dataHeader,
    const QString&	dataName,
    const QString&	dataBaseName,
    bool		isDialog,
    bool		isDialogModal,
    WFlags		windowFlags
    );
  // generate the dialog data source file

    
  //
  // Cut and Paste support
  //
  bool CopySelected( QtArchGenerator & );
  bool CutSelected( QtArchGenerator & );

  enum ScreenFlags
  {
    SF_None			= 0x0000,
    SF_WidgetSelected	= 0x0001,
    SF_Align		= 0x0002
  };
  // Flags used to set the availability of operations through the menus
  // and toolbars


  //
  // Property Accessors
  //
  QSize GetWindowSize( void ) const;
  void SetWindowSize( const QSize&	theSize );
  // Get/Set methods on the window's size

  const QSize& GetWindowMinSize( void ) const;
  void SetWindowMinSize( const QSize&	theSize );
  // Get/Set methods on the window's min size

  const QSize& GetWindowMaxSize( void ) const;
  void SetWindowMaxSize( const QSize&	theSize );
  // Get/Set methods on the window's max size

  const QPoint& GetWindowPos( void ) const;
  void SetWindowPos( const QPoint&	thePoint );
  // Get/Set methods on the window's initial position, -1,-1 for no initial
  // position

  bool IsTextTranslated() const;
  void SetTextTranslated( bool isTranslated );
  // Get/Set methods for the use of tr() to translate text
    

  //
  // Miscellaneous
  //
  void InitialWindowResize();
  // HACK needed to get the window to display with the correct size

  void AutoSelect( bool = TRUE );
  // Automatically select all new widgets


  //
  // Event Filter for widgets
  //
  bool eventFilter(
    QObject*	theSender,
    QEvent*		theEvent
    );
  // Event filter for the window widgets.

  void GetAllSlots(
    QDict<QString> *publicSlots, 
    QDict<QString> *protectedSlots, 
    const QString &dataBaseName
    );
  // Extracts all the defined slots for the class, used when generating code

  signals:

  void Changed();

  void LayoutWndClosed();
  // The layout window was closed

  void WidgetGeometryChanged( int, int, int, int );
  // emit when a widget's geometry has changed with its x and y coordinate
  // and width and height

  void WidgetSelected( const QString&, const QString& );
  // emit when a widget is selected with the widget's name and 
  //mapped variable
    
  void WidgetDeselected();
  // emit when all widgets where deselected (Mouse button 2)

  void UpdateScreen( int );
  // Tell the dialog window to update the screen
    
  public slots: // methods

    void UpdateWidgetLayoutState( const DlgWidget*	theDlgWidget );
  // Update the layout status of the widget.

  void RaiseSelectedWidget();

  void LowerSelectedWidget();

  void DeleteSelectedWidget();
    
  void SelectedWidgetProperties();
  void OptionsGrid();

  void ContentsRestored();
  // All Widgets are restored, disconnect from Parser

  void AlignTop();
  // align selected widgets to top of first widget

  void AlignBottom();
  // align selected widgets to bottom of first widget

  void AlignLeft();
  // align selected widgets to left of first widget

  void AlignRight();
  // align selected widgets to right of first widget

  void SizeHorizontal();
  // give selected widgets the same width as the first widget

  void SizeVertical();
  // give selected widgets the same height as the first widget

  void SizeBothHV();
  // give selected widgets the same width and height as the first widget

  void SizeToFit();
  // Resizes the widget to its content's size

  void MoveUp1();
  // Move the selected widgets up one grid spacing

  void MoveDown1();
  // Move the selected widgets down one grid spacing

  void MoveLeft1();
  // Move the selected widgets left one grid spacing

  void MoveRight1();
  // Move the selected widgets right one grid spacing

  void ResizeShorter1();
  // Resize the selected widgets one grid spacing shorter

  void ResizeWider1();
  // Resize the selected widgets one grid spacing widget

  void ResizeNarrower1();
  // Resize the selected widgets one grid spacing narrower

  void ResizeTaller1();
  // Resize the selected widgets one grid spacing taller

  void MoveUpMultiple();
  // Move the selected widgets up multiple grid spacings
 
  void MoveDownMultiple();
  // Move the selected widgets down multiple grid spacings
 
  void MoveLeftMultiple();
  // Move the selected widgets left multiple grid spacings
 
  void MoveRightMultiple();
  // Move the selected widgets right multiple grid spacings
 
  void ResizeShorterMultiple();
  // Resize the selected widgets multiple grid spacings shorter
 
  void ResizeWiderMultiple();
  // Resize the selected widgets multiple grid spacings widget
 
  void ResizeNarrowerMultiple();
  // Resize the selected widgets multiple grid spacings narrower
 
  void ResizeTallerMultiple();
  // Resize the selected widgets multiple grid spacings taller
 
  private: // Attributes

  struct Selected
  {
    Selected( 
	    QWidget*		theWidget,
	    WidgetFrame*	theWidgetFrame = 0,
	    bool		isMoveable = FALSE
	    );

    Selected(
	    DlgWidget&		theWidget,
	    WidgetFrame*	theWidgetFrame
	    );

    ~Selected();

    QWidget *widget;
    QPoint   newPos;
    SelectedWidgetBorder* myWidgetBorder;
  };

  class SelectList : public QList<Selected>
    {
      protected:
      int compareItems( Item i1, Item i2 )
        { return (((const Selected*)i1)->widget != 
                  ((const Selected*)i2)->widget ); };
    };

  private slots: // methods

    void RestoreLayoutKeyValue( QString&	key, 
                                QString&	value );        
  // Restore a single key/value pair for the Layout 

  void RestoreNewWidget( 
    QString &widgetName 
    );
  // Restore a new Widget

  void RestoreWidgetKeyValue( 
    QString &key, 
    QString &value 
    );        
  // Restore a single key/value pair for a Widget

  void RestoreEndWidget();        
  // Constructing a new widget is finished

  private: // methods

  //
  // New widget
  //

  void AddWidget(
    DlgWidget*	theNewWidget,
    bool		isWidgetOnTop = TRUE
    );

  //
  // Process events on the widgets
  //

  void ProcessMousePress(
    QWidget* 	widget,
    QMouseEvent*	event
    );
    
  void ProcessMouseMove(	
    QWidget*	widget,
    QMouseEvent*	event
    );
    
  QRect CalculateGeometryRect(
    QWidget*	widget,
    QPoint		pos
    );
  // calculate the new geometry rect for the widget


  DlgWidget* GetDlgWidget(
    const QWidget*	widget 
    );
  // given a widget, return its DlgWidget

    
  //
  // Selected Widget Manipulators
  //
  void SetSelected(
    QWidget*	widget,
    bool 		selected
    );

  void AddToSelection( QWidget * );

  void RemoveFromSelection( QWidget * );

  bool IsSelected( QWidget*	widget );


  void SetDragging(
    QWidget*	widget
    );

  void UpdateCursor
    (
      QWidget*	widget
      );

  void UpdateNoMoveWidgets( void );
  // Update the list of widgets that can't be moved.

  void MoveWidget( const QPoint&	newPos );
  // move the widget be the new position relative to the top left corner

  void ResizeWidget( QWidget*		widget,
                     const QRect&	newRect );
  // Resize the widget to the specified rect

  void WidgetGeometryChange( QWidget* widget );
  // process a change in widget geometry

  void ConnectToWidget( QWidget* widget );
  // connect the necessary events from the widget to the slots for 
  // manipulating the widget

  void AddScreenFlags( int flags );
  // Add the given flags to the set of flags and update the screen

  void RemoveScreenFlags( int flags );
  // Remove the given flags from the set of flags and update the screen


  //
  // Code Generation
  //

  void GenerateSourceForChildren(     
    QWidget*        parent,
    QTextStream&    stream, 
    QDict<char>&    visitedChildren
    );
  // Generate source for all children of the given parent widget

  void GenerateWidgetSource(
    DlgWidget*		widget,
    QTextStream&		stream
    );
  // generate the source for one widget

  void MoveUp(int multiple);
  // Move the selected widgets up

  void MoveDown(int multiple);
  // Move the selected widgets down

  void MoveLeft(int multiple);
  // Move the selected widgets left

  void MoveRight(int multiple);
  // Move the selected widgets right

  void ResizeShorter(int multiple);
  // Resize the selected widgets x grid spacings shorter

  void ResizeWider(int multiple);
  // Resize the selected widgets x grid spacings widget

  void ResizeNarrower(int multiple);
  // Resize the selected widgets x grid spacings narrower

  void ResizeTaller(int multiple);
  // Resize the selected widgets x grid spacings taller
    
  private: // attributes

  enum RestoreAttr
  {
    APPLICATION,
    LAYOUT,
    WIDGET
  };

  RestoreAttr attrType;
  // The type of the actual read attribute
  
  DialogWnd *itsParent;
  // The application
  
  WidgetFrame*	itsDlgEditWnd;
  // window on which the widgets are created and manipulated

  LayoutHelper	itsLayoutHelper;
  // Helper class for handling of the layout
    
  QPopupMenu* 	itsWidgetPopup;
  // popup menu for widgets

  QPopupMenu*		itsWidgetFramePopup;
  // popup menu for the widget frame
    
  SelectList		itsSelectedWidget;
  // List of actual selected Widgets

  QPoint		itsWidgetClick;
  bool		isDragging;

  QDict< DlgWidget >	itsDlgWidgets;
  // dictionary of dlgwidgets indexed by the name of the widget

  QDict< QWidget >	itsNoMoveWidgets;
  // widgets whose top left corner doesn't move

  QList< DlgWidget >	itsZOrdering;
  // z ordering of the widgets on the screen.  index 0 is the bottommost
  // widget

  DlgParser *itsParser;
  // The Parser which will be used to read the dilaog file

  DlgWidget *widgetInConstruction;
  // The actual Widget which is just build

  bool autoSelect;
  // Automatically select all new widgets

  int itsScreenFlags;
  // the flags used to determine which screen elements are enabled

  QSize itsWindowMinSize;
  QSize itsWindowMaxSize;
  // Min and Max sizes of the window
    
  QPoint itsWindowPos;
  // The initial position of the window, -1,-1 for no initial pos

    QSize itsInitialSize;
    // Initial size of the window.  Needed as a hack to get the window to
    // initially display at the correct size.

    bool isTextTranslated;
    // Is text translated with tr()
};


inline void WidgetLayout::Start( void )
{
    emit UpdateScreen( itsScreenFlags );
}


inline void WidgetLayout::SetCaption(
    const QString&	theCaption
    )
{
    itsLayoutHelper.SetCaption( theCaption );
}


inline const DlgWidget* WidgetLayout::GetWidget
(
    const QString&	name
) const
{
    return itsDlgWidgets[ name ];
}


inline WidgetFrame *WidgetLayout::GetDlgEditWnd() const
{
    return itsDlgEditWnd;
}

inline void WidgetLayout::SetNoMove
(
    QWidget*	widget
)
{
    itsNoMoveWidgets.insert( widget->name(), widget );
}


inline void WidgetLayout::AutoSelect( bool aS )
{
    autoSelect = aS;
}


inline void WidgetLayout::AddScreenFlags( int flags )
{
    itsScreenFlags |= flags;
    emit UpdateScreen( itsScreenFlags );
}


inline void WidgetLayout::RemoveScreenFlags( int flags )
{
    itsScreenFlags &= ~flags;
    emit UpdateScreen( itsScreenFlags );
}


inline QSize WidgetLayout::GetWindowSize( void ) const
{
    return itsDlgEditWnd->size();
}


inline void WidgetLayout::SetWindowSize( const QSize&	theSize )
{
    itsDlgEditWnd->setFixedSize( theSize );
}


inline const QSize& WidgetLayout::GetWindowMinSize( void ) const
{
    return itsWindowMinSize;
}


inline void WidgetLayout::SetWindowMinSize( const QSize&	theSize )
{
    itsWindowMinSize = theSize;
}


inline const QSize& WidgetLayout::GetWindowMaxSize( void ) const
{
    return itsWindowMaxSize;
}


inline void WidgetLayout::SetWindowMaxSize( const QSize&	theSize )
{
    itsWindowMaxSize = theSize;
}


inline const QPoint& WidgetLayout::GetWindowPos( void ) const
{
    return itsWindowPos;
}


inline void WidgetLayout::SetWindowPos( const QPoint&	thePoint )
{
    itsWindowPos = thePoint;
}


inline void WidgetLayout::InitialWindowResize()
{
    itsDlgEditWnd->setFixedSize( itsInitialSize );
}


inline void WidgetLayout::AddToSelection( const DlgWidget* widget )
{
    AddToSelection( widget->GetWidget() );
}


inline bool WidgetLayout::IsSelected( const DlgWidget* widget )
{
    return IsSelected( widget->GetWidget() );
}


inline bool WidgetLayout::IsTextTranslated() const
{
    return isTextTranslated;
}


inline void WidgetLayout::SetTextTranslated( bool isTranslated )
{
    isTextTranslated = isTranslated;
}



#endif // WidgetLayout_included



