/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetModuleMgr.cpp,v 2.6 2000/04/05 00:35:03 jeff Exp $

*******************************************************************************/


#include <qstrlist.h>

#include "module/WidgetModule.h"

#include "ProjectWnd.h"
#include "WidgetModuleMgr.h"

#ifdef _WIN32

#elif __hpux

#include <dl.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/*========================================================================*
 * TITLE:       libQmec
 * PORT:        Ross Linder     [Mecalc Pty Ltd]
 *
 * Interface for stupid non ELF OS's
 *========================================================================

 * This is a minimal implementation of the ELF dlopen, dlclose, dlsym
 * and dlerror routines based on HP's shl_load, shl_unload and
 * shl_findsym. 
 */

/*
 * Reference Counting.
 *
 * Empirically it looks like the HP routines do not maintain a
 * reference count, so I maintain one here.
 */

typedef struct lib_entry {
    shl_t handle;
    int count;
    struct lib_entry *next;
} *LibEntry;

#define lib_entry_handle(e) ((e)->handle)
#define lib_entry_count(e) ((e)->count)
#define lib_entry_next(e) ((e)->next)
#define set_lib_entry_handle(e,v) ((e)->handle = (v))
#define set_lib_entry_count(e,v) ((e)->count = (v))
#define set_lib_entry_next(e,v) ((e)->next = (v))
#define increment_lib_entry_count(e) ((e)->count++)
#define decrement_lib_entry_count(e) ((e)->count--)

static LibEntry Entries = NULL;

static LibEntry find_lib_entry(shl_t handle)
{
    LibEntry entry;
    
    for (entry = Entries; entry != NULL; entry = lib_entry_next(entry))
        if (lib_entry_handle(entry) == handle)
            return entry;
    return NULL;
}

static LibEntry new_lib_entry(shl_t handle)
{
    LibEntry entry;


    if ((entry = (LibEntry) malloc(sizeof(struct lib_entry))) != NULL) {
        set_lib_entry_handle(entry, handle);
        set_lib_entry_count(entry, 1);
        set_lib_entry_next(entry, Entries);
        Entries = entry;
    }
    return entry;
}

static void free_lib_entry(LibEntry entry)
{
    if (entry == Entries)
        Entries = lib_entry_next(entry);
    else {
        LibEntry last, next;
        for (last = Entries, next = lib_entry_next(last);
             next != NULL;
             last = next, next = lib_entry_next(last)) {
            if (entry == next) {
                set_lib_entry_next(last, lib_entry_next(entry));
                break;
            }
        }
    }
    free(entry);
}


/*
 * Error Handling.
 */

#define ERRBUFSIZE 1000

static char errbuf[ERRBUFSIZE];
static int dlerrno = 0;

char *dlerror(void)
{
    return dlerrno ? errbuf : NULL;
}


/*

 * Opening and Closing Liraries.
 */

void *dlopen(const char *fname, int mode)
{
    shl_t handle;
    LibEntry entry = NULL;
  
    dlerrno = 0;
    if (fname == NULL)
        handle = PROG_HANDLE;
    else {
        handle = shl_load(fname, mode, 0L);
        if (handle != NULL) {
            if ((entry = find_lib_entry(handle)) == NULL) {
                if ((entry = new_lib_entry(handle)) == NULL) {
                    shl_unload(handle);
                    handle = NULL;
                }
            }
            else
                increment_lib_entry_count(entry);
        }
        if (handle == NULL) {
            char *errstr;
            dlerrno = errno;
            errstr = strerror(errno);
            if (errno == NULL) errstr = "???";
            sprintf(errbuf, "can't open %s: %s", fname, errstr);
        }
    }
#ifdef DEBUG
    printf("opening library %s, handle = %x, count = %d\n",
           fname, handle, entry ? lib_entry_count(entry) : -1);
    if (dlerrno) printf("%s\n", dlerror());
#endif
    return (void *) handle;
}


int dlclose(void *handle)
{
    LibEntry entry;
#ifdef DEBUG
    entry = find_lib_entry(handle);
    printf("closing library handle = %x, count = %d\n",
           handle, entry ? lib_entry_count(entry) : -1);
#endif

    dlerrno = 0;
    if ((shl_t) handle == PROG_HANDLE)
        return 0; /* ignore attempts to close main program */
    else {

        if ((entry = find_lib_entry((shl_t) handle)) != NULL) {
            decrement_lib_entry_count(entry);
            if (lib_entry_count(entry) > 0)
                return 0;
            else {
                /* unload once reference count reaches zero */
                free_lib_entry(entry);
                if (shl_unload((shl_t) handle) == 0)
                    return 0;
            }
        }
        /* if you get to here, an error has occurred */
        dlerrno = 1;
        sprintf(errbuf, "attempt to close library failed");
#ifdef DEBUG
        printf("%s\n", dlerror());
#endif
        return -1;
    }
}


/*
 * Symbol Lookup.
 */

void *dlsym(void *handle, const char *name)
{
    void *f;
    shl_t myhandle;

    dlerrno = 0;
    myhandle = (handle == NULL) ? PROG_HANDLE : (shl_t) handle;

    if (shl_findsym(&myhandle, name, TYPE_PROCEDURE, &f) != 0) {
        dlerrno = 1;
        sprintf(errbuf, "symbol %s not found", name);
        f = NULL;
    }

    return(f);
}

#else

#include <dlfcn.h>

#endif

WidgetModuleMgr WidgetModuleMgr::itsInstance;


static const int MODULE_MAJOR_VERSION = 1;
static const int MODULE_MINOR_VERSION = 0;


WidgetModuleMgr::WidgetModuleMgr()
{
    myModuleHandles.setAutoDelete( TRUE );
    myWidgetModules.setAutoDelete( TRUE );
}


WidgetModuleMgr::~WidgetModuleMgr()
{
    myWidgetModules.clear();
    
    QListIterator< WidgetModuleHandle > iter( myModuleHandles );
    WidgetModuleHandle* handle = 0;
    while( (handle = iter()) )
    {
	unloadModule( *handle );
    }
    
}


void WidgetModuleMgr::_loadModules()
{
    QStrList moduleNames;
    
    getModuleNames( moduleNames );

    for( unsigned int i = 0; i < moduleNames.count(); ++i )
    {
	WidgetModuleHandle handle;
	if( loadModule( moduleNames.at( i ),
			handle ) )
	{
	    WidgetModule* widgetModule = 0;
	    
	    if( (widgetModule = getWidgetModule( handle )) != 0 )
	    {
		myModuleHandles.append( new WidgetModuleHandle( handle ) );
		myWidgetModules.append( widgetModule );
		myWidgetModuleDict.insert( widgetModule->getName(), 
					   widgetModule );

		cout << 
                    QObject::tr( "Module %1 loaded" ).
                    arg( widgetModule->getName() )
		     << endl;
	    }
	}
    }	
}


DlgWidget* WidgetModuleMgr::_newWidget(
    int		theModuleNum,
    int		theModuleWidgetNum,
    QWidget*	theParent
    )
{
    DlgWidget* dlgWidget = 0;
    
    WidgetModule* widgetModule = myWidgetModules.at( theModuleNum );
    if( widgetModule )
    {
	dlgWidget = widgetModule->newWidget( theModuleWidgetNum );

	if( dlgWidget )
	{
            dlgWidget->SetParent( theParent );
	}
    }
    else
    {
	cerr << QObject::tr( "No module for index: %1" ).arg( theModuleNum )
             << endl;
    }
    
    return dlgWidget;
}


DlgWidget* WidgetModuleMgr::_newWidget(
    const QString&	theModuleName,
    const QString&	theModuleWidgetName,
    QWidget*		theParent
    )
{
    DlgWidget* dlgWidget = 0;
    
    WidgetModule* widgetModule = myWidgetModuleDict[ theModuleName ];
    if( widgetModule )
    {
	dlgWidget = widgetModule->newWidget( theModuleWidgetName );
	
	if( dlgWidget )
	{
            dlgWidget->SetParent( theParent );
	}
	else
	{
	    cerr << QObject::tr( "No widget for module %1: %2" ).
                arg( theModuleName ).arg( theModuleWidgetName ) << endl;
	}
    }
    else
    {
	cerr << QObject::tr( "No module for name: %1" ).arg( theModuleName ) 
             << endl;
    }

    return dlgWidget;
}


typedef QApplication* (*GetQApplication_Type) (int&,char**);

QApplication* WidgetModuleMgr::_getQApplication(
    int&	theArgc,
    char**	theArgv
    )
{
    QApplication* qapp = 0;
    
    long qappModule = ProjectWnd::GetAppConfig().readEntry( "QAppModule", 
							    (long) -1 );
    
    if( qappModule > -1 )
    {
	WidgetModuleHandle* handle = myModuleHandles.at( qappModule );
	if( handle )
	{
	    GetQApplication_Type GetQApplication =
		(GetQApplication_Type)loadFunction( *handle, 
						    "GetQApplication" );
	    
	    if( GetQApplication )
	    {
		qapp = (*GetQApplication)( theArgc, theArgv );
	    }
	}
	else
	{
	    cerr << QObject::tr( "No handle for QApp module: %1" ).arg( qappModule )
                 << endl;
	}
    }
    
    return qapp;
}


void WidgetModuleMgr::getModuleNames(
    QStrList&	theNames
    )
{
    AppConfig& appConfig = ProjectWnd::GetAppConfig();
    appConfig.setCurrentPath( "WidgetModules" );
    
    BaseConfig::Enumerator* moduleEnum = appConfig.enumEntries();
    
    for( unsigned int i = 0; i < moduleEnum->Count(); ++i )
    {
	theNames.append( appConfig.readEntry( (*moduleEnum)[i] ) );
    }
    
    delete moduleEnum;

    appConfig.setCurrentPath( "/" );
}

typedef void (*GetModuleVersion_Type) (int&, int&);

bool WidgetModuleMgr::loadModule(
    const QString&	theModuleName,
    WidgetModuleHandle&	theModuleHandle
    )
{
    bool rc = FALSE;

#ifdef _WIN32

    HMODULE handle = LoadLibrary( theModuleName );
    if( handle )
    {
        theModuleHandle = handle;
        rc = TRUE;
    }
    else
    {
        cerr << QObject::tr( "Failed to load module %1: %2" ).
            arg( theModuleName ).arg( GetLastError() ) << endl;
    }
    

#else
    
    void* handle = dlopen( theModuleName, RTLD_LAZY );
    if( handle != NULL )
    {
	theModuleHandle = handle;
	rc = TRUE;
    }
    else
    {
        cerr << QObject::tr( "Failed to load module %1: %2" ).
            arg( theModuleName ).arg( dlerror() ) << endl;
    }

#endif

    if( rc )
    {
	// Check the version of the module
	GetModuleVersion_Type GetModuleVersion = 
	    (GetModuleVersion_Type)loadFunction( theModuleHandle,
	 					 "GetModuleVersion" );
						 
	if( GetModuleVersion )
	{
	    int majorVersion;
	    int minorVersion;
	    (*GetModuleVersion)( majorVersion, minorVersion );
    
	    if( majorVersion != MODULE_MAJOR_VERSION )
	    {
		cerr << 
                    QObject::tr( "Error: Module major versions do not match. QtArch: %1, module: %2" ).
                    arg( MODULE_MAJOR_VERSION ).arg( majorVersion )
                     << endl;

                unloadModule( theModuleHandle );
		rc = FALSE;
	    }

	    if( minorVersion != MODULE_MINOR_VERSION )
	    {
		cerr << 
                    QObject::tr( "Warning: Module minor versions do not match. QtArch: %1, module: %2" ).
                    arg( MODULE_MINOR_VERSION ).arg( minorVersion )
                     << endl;
	    }
	}
	else
	{
	    cerr << QObject::tr( "Failed to load function GetModuleVersion" ) << endl;
	}
    }

    return rc;
}


void WidgetModuleMgr::unloadModule(
    WidgetModuleHandle	theHandle
    )
{
#ifdef _WIN32

    FreeLibrary( theHandle );

#else

    dlclose( theHandle );

#endif
}


typedef WidgetModule* (*GetWidgetModule_Type) ();

WidgetModule* WidgetModuleMgr::getWidgetModule(
    WidgetModuleHandle	theHandle
    )
{
    WidgetModule* widgetModule = 0;
    
    GetWidgetModule_Type GetWidgetModule = 
	(GetWidgetModule_Type)loadFunction( theHandle, "GetWidgetModule" );

    if( GetWidgetModule != NULL )
    {
	widgetModule = (*GetWidgetModule)();

	if( !widgetModule )
	{
	    cerr << QObject::tr( "No widget module returned" ) << endl;
	}
    }

    return widgetModule;
}


void* WidgetModuleMgr::loadFunction(
    WidgetModuleHandle	theHandle,
    const char*		theFunctionName
    )
{
    void* fcn = 0;

#ifdef _WIN32

    fcn = (void*) GetProcAddress( theHandle, theFunctionName );
    
    if( fcn == NULL )
    {
        cerr << QObject::tr( "Failed to load function %1: %2" ).
            arg( theFunctionName ).arg( GetLastError() ) << endl;
    }

#else

    fcn = dlsym( theHandle, theFunctionName );
    
    if( fcn == NULL )
    {
        cerr << QObject::tr( "Failed to load function %1: %2" ).
            arg( theFunctionName ).arg( dlerror() ) << endl;
    }
    
#endif

    return fcn;
}
