/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridLayout.h,v 2.6 2000/04/29 20:15:46 klaus Exp $

*******************************************************************************/


#ifndef GridLayout_included
#define GridLayout_included

#include <qlist.h>

#include "layout/LayoutItem.h"

class DlgMenuBar;
class QtArchGenerator;

class GridLayout : public LayoutItem
{
public: // Types

    struct GridColumnInfo
    {
	GridColumnInfo( int	spacing,
			int 	stretch );

	~GridColumnInfo();

	int mySpacing;
	int myStretch;
    };


public: // Methods

    GridLayout(
	LayoutItem*	theParent
	);
    // Constructor

    GridLayout(
	QListView*	theParent
	);
    // Constructor

    virtual ~GridLayout();
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout );
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    //
    // Layout creation
    //

    virtual QLayout* RecreateLayout( QWidget*	theBaseWindow );
    // Create a new layout for theBaseWindow and add all of its children to it.

    virtual void AddToLayout( QBoxLayout*	theBoxLayout );
    // Add the item and all its children to a box layout.

    void UpdateColumnInfo();
    // Update the current grid column info.



    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.
    
    virtual void Restore( const XMLValue &, LayoutHelper * );
    // Restore the item from the given XMLValue
    
    virtual bool RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget );
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.


    //
    // Code generation
    //

    virtual void GenerateSource( QTextStream&	theStream,
				 const QString&	theLayoutId = "1",
				 const QString&	theLayoutParentWidget = QString() ) const;
    // Generate the source for the entire layout tree.

    virtual void GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	theLayoutId,
				      const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a box layout.


    virtual void GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout.


    //
    // Get/Set Properties
    //

    // Border
    int GetBorder( void ) const;
    void SetBorder( int	border );
    
    // Auto Border
    int GetAutoBorder( void ) const;
    void SetAutoBorder( int	autoBorder );
    
    // Name
    const QString& GetName( void ) const;
    void SetName( const QString&	name );
    
    // MenuBar widget
    const DlgMenuBar* GetMenuBarWidget( void ) const;
    void SetMenuBarWidget( const DlgMenuBar*	menuWidget );

    // Column Information
    QList<GridColumnInfo>& GetColumnInfo( void );

    // Box Stretch
    int GetBoxStretch( void ) const;
    void SetBoxStretch( int	theBoxStretch );

private: // Methods

    void SetLayoutContents( QGridLayout*	theLayout );
    // Set the properties and the children of the layout.

    void GenerateCtor( QTextStream&	theStream,
		       const QString&	theVarName,
		       bool		isTopLevel,
		       const QString&	theLayoutParentWidget ) const;
    // Generate the constructor for the grid layout.

    void GenerateContents( QTextStream&		theStream,
			   const QString&	theVarName,
			   const QString&	theLayoutId,
			   const QString&	theLayoutParentWidget ) const;
    // Generate the contents of the grid layout

private: // Class Methods

    static QString GetTreeLabel();
    // Get the label for the item in the tree
    
private: // Attributes

    //
    // Properties
    //
    
    int					myBorder;
    int					myAutoBorder;
    QString				myName;
    const DlgMenuBar*			myMenuBarWidget;
    QList<GridColumnInfo>	        myColumnInfo;
    int					myBoxStretch;
};


inline int GridLayout::GetBorder( void ) const
{
    return myBorder;
}


inline void GridLayout::SetBorder( int	border )
{
    myBorder = border;
}


inline int GridLayout::GetAutoBorder( void ) const
{
    return myAutoBorder;
}


inline void GridLayout::SetAutoBorder( int	autoBorder )
{
    myAutoBorder = autoBorder;
}


inline const QString& GridLayout::GetName( void ) const
{
    return myName;
}


inline const DlgMenuBar* GridLayout::GetMenuBarWidget( void ) const
{
    return myMenuBarWidget;
}


inline void GridLayout::SetMenuBarWidget( const DlgMenuBar*	menuWidget )
{
    myMenuBarWidget = menuWidget;
}


inline QList<GridLayout::GridColumnInfo>& GridLayout::GetColumnInfo( void )
{
    return myColumnInfo;
}


inline int GridLayout::GetBoxStretch( void ) const
{
    return myBoxStretch;
}


inline void GridLayout::SetBoxStretch( int	theBoxStretch )
{
    myBoxStretch = theBoxStretch;
}


#endif // GridLayout_included
