/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgGrid.cpp,v 2.8 2000/04/30 17:09:46 jeff Exp $

*******************************************************************************/


#include "widget/DlgGrid.h"

#include "prop/GridPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgGrid::itsSignals;
int DlgGrid::itsNextWidgetNum = 1;

DlgGrid::DlgGrid() :
    Inherited( DlgWidget::DW_Grid ),
    myDirection( QGrid::Horizontal ),
    myNumRowsCols( 5 ),
    mySpacing( 5 )
{
}


DlgGrid::DlgGrid(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    myDirection( QGrid::Horizontal ),
    myNumRowsCols( 5 ),
    mySpacing( 5 )
{
}


DlgGrid::~DlgGrid()
{
}


void DlgGrid::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new GridPage( this, parent, "Grid" ) );

    labels.append( tr( "Grid" ) );
}

void DlgGrid::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "Grid" );

  generator.WriteSingleLine( "Direction", DirectionToStr( myDirection ) );
  generator.WriteXMLValue( "RowCols", myNumRowsCols );
  generator.WriteXMLValue( "Spacing", mySpacing );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgGrid::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    // Compare to known keys
    if( key == "Direction" )
    {
        SetDirection( StrToDirection( value ) );
    }
    else if( key == "RowsCols" )
    {
        SetNumRowsCols( value.toInt() );
    }
    else if( key == "Spacing" )
    {
        SetSpacing( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgGrid::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "Grid" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetGrid() )
  {
    CreateQtWidget( 0 );
  }
  
  SetDirection( StrToDirection( map["Direction"].AsQString() ) );
  SetNumRowsCols( map["RowCols"].AsInt() );
  SetSpacing( map["Spacing"].AsInt() );

  // We inherit from 
	Inherited::RestoreXML( map["Frame"] );

}


void DlgGrid::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    stream << varName << "setSpacing( " << mySpacing << " );" << endl;
}


void DlgGrid::GenerateCtor(
    QTextStream&	stream,
    const QString&	parentName
    )
{
    QString name;
    const char* cp = GetWidget()->name();
    if( cp )
    {
	name = "\"";
	name += cp;
	name += "\"";
    }
    else
    {
	name = "NULL";
    }

    stream << myNumRowsCols << ", " 
           << "QGrid::" << DirectionToStr( myDirection ) << ", "
           << parentName << ", " << name;
}


void DlgGrid::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgGrid::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


QString DlgGrid::DirectionToStr( QGrid::Direction theDirection )
{
    QString str;
    
    switch( theDirection )
    {
        case QGrid::Horizontal:
        {
            str = "Horizontal";
            break;
        }
        case QGrid::Vertical:
        {
            str = "Vertical";
            break;
        }
        default:
        {
            break;
        }
    }
    
    return str;
}


QGrid::Direction DlgGrid::StrToDirection( const QString& theDirection )
{
    QGrid::Direction dir = QGrid::Horizontal;
    
    if( theDirection == "Horizontal" )
    {
        dir = QGrid::Horizontal;
    }
    else if( theDirection == "Vertical" )
    {
        dir = QGrid::Vertical;
    }
    
    return dir;
}


void DlgGrid::CreateQtWidget(
    const char*	theName
    )
{
    QGrid* widget = 
	new QGrid( myNumRowsCols,
                   myDirection,
                   GetParent(),
                   CreateName( "Grid", 
                               theName, 
                               itsNextWidgetNum ) );

    SetQtWidget( widget );

    SetSpacing( mySpacing );
}
