/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSlider.cpp,v 2.7 2000/04/30 17:09:48 jeff Exp $

*******************************************************************************/


#include "widget/DlgSlider.h"

#include "prop/SliderPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgSlider::itsSignals;
int DlgSlider::itsNextWidgetNum = 1;

DlgSlider::DlgSlider() :
    Inherited( DlgWidget::DW_Slider )
{
}


DlgSlider::DlgSlider(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgSlider::~DlgSlider()
{
}


void DlgSlider::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SliderPage( this, parent, "Slider" ) );

    labels.append( tr( "Slider" ) );
}

void DlgSlider::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "Slider" );
  
  QSlider* widget = GetSlider();

    QString orientation;
  if( widget->orientation() == QScrollBar::Horizontal )
  {
    orientation = "Horizontal";
  }
  else
  { 
    orientation = "Vertical";
  }
  generator.WriteSingleLine( "Orientation", orientation );

  generator.WriteXMLValue( "MinValue", widget->minValue() );
  generator.WriteXMLValue( "MaxValue", widget->maxValue() );
  generator.WriteXMLValue( "Initial", widget->value() );
  generator.WriteXMLValue( "LineStep", widget->lineStep() );
  generator.WriteXMLValue( "PageStep", widget->pageStep() );
  generator.WriteXMLValue( "Tracking", widget->tracking() );

  generator.WriteSingleLine( "Tickmarks", 
                             TickSettingToStr( widget->orientation(),	
                                               widget->tickmarks() ));
  
  generator.WriteXMLValue( "TickInterval", widget->tickInterval() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgSlider::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QSlider* widget = GetSlider();

    // Compare to known keys
    if( key == "Orientation" )
    {
	if( value == "Horizontal" )
	{
	    widget->setOrientation( QSlider::Horizontal );
	}
	else
	{
	    widget->setOrientation( QSlider::Vertical );
	}
    } 
    else if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    }
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Initial" )
    {
	widget->setValue( value.toInt() );
    }
    else if( key == "LineStep" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "PageStep" )
    {
	widget->setSteps( widget->lineStep(), value.toInt() );
    }
    else if( key == "Tracking" )
    {
	widget->setTracking( (value == "TRUE") );
    }
    else if( key == "Tickmarks" )
    {
	widget->setTickmarks( StrToTickSetting( value ) );
    }
    else if( key == "TickInterval" )
    {
	widget->setTickInterval( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgSlider::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "Slider" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetSlider() )
    {
        CreateQtWidget( 0 );
    }
  
    QSlider* widget = GetSlider();

    Qt::Orientation orientation;
    if( map["Orientation"].AsQString() == "Horizontal" )
        orientation = Qt::Horizontal;
    else
        orientation = Qt::Vertical;
    widget->setOrientation( orientation );

    widget->setRange( map["MinValue"].AsInt(), map["MaxValue"].AsInt() );
    widget->setValue( map["Initial"].AsInt() );
    widget->setSteps( map["LineStep"].AsInt(), map["PageStep"].AsInt() );
    widget->setTracking( map["Tracking"].AsBool() );
    widget->setTickmarks( StrToTickSetting( map["Tickmarks"].AsQString() ) );
    widget->setTickInterval( map["TickInterval"].AsInt() );

    // We inherit from 
    Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgSlider::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QSlider* widget = GetSlider();

    stream << varName << "setOrientation( QSlider::" <<
	((widget->orientation() == QSlider::Horizontal) ? "Horizontal" : "Vertical") <<
	" );" << endl;
    
    stream << varName << "setRange( " <<
	widget->minValue() << ", " << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( " <<
	widget->lineStep() << ", " << widget->pageStep() << " );" << endl;
    
    stream << varName << 
	"setValue( " << widget->value() << " );" << endl;
    
    stream << varName << "setTracking( " <<
	(widget->tracking() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setTickmarks( QSlider::" <<
	TickSettingToStr( widget->orientation(),
                          widget->tickmarks() ) << " );" << endl;
    
    stream << varName <<
	"setTickInterval( " << widget->tickInterval() << " );" << endl;
}


void DlgSlider::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSlider::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "sliderPressed()" );
	itsSignals.append( "sliderMoved(int)" );
	itsSignals.append( "sliderReleased()" );
    }

    return itsSignals;
}


QString DlgSlider::TickSettingToStr( QSlider::Orientation       orient,
                                     QSlider::TickSetting       tick )
{
    QString str = "NoMarks";
    

    if( orient == QSlider::Vertical )
    {
	if( tick == QSlider::Left )
	{
	    str = "Left";
	}
	else if( tick == QSlider::Right )
	{
	    str = "Right";
	}
    }
    else if( orient == QSlider::Horizontal )
    {
	if( tick == QSlider::Above )
	{
	    str = "Above";
	}
	else if( tick == QSlider::Below )
	{
	    str = "Below";
	}
    }

    if( tick == QSlider::Both )
    {
	str = "Both";
    }
    
    return str;
}


QSlider::TickSetting DlgSlider::StrToTickSetting( const QString& str )
{
    QSlider::TickSetting tick = QSlider::NoMarks;
    
    if( str == "Above" )
    {
	tick = QSlider::Above;
    }
    else if( str == "Left" )
    {
	tick = QSlider::Left;
    }
    else if( str == "Below" )
    {
	tick = QSlider::Below;
    }
    else if( str == "Right" )
    {
	tick = QSlider::Right;
    }
    else if( str == "Both" )
    {
	tick = QSlider::Both;
    }
    
    return tick;
}


void DlgSlider::CreateQtWidget(
    const char*	theName
    )
{
    QSlider* widget = 
	new QSlider( QSlider::Horizontal,
		     GetParent(),
		     CreateName( "Slider", 
				 theName, 
				 itsNextWidgetNum ) );

    SetQtWidget( widget );
}
