/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgTabBar.cpp,v 2.7 2000/04/30 17:09:49 jeff Exp $

*******************************************************************************/


#include "widget/DlgTabBar.h"

#include "prop/TabBarPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgTabBar::itsSignals;
int DlgTabBar::itsNextWidgetNum = 1;

DlgTabBar::DlgTabBar() :
    Inherited( DlgWidget::DW_TabBar )
{
    myLabelList.setAutoDelete( TRUE );
}


DlgTabBar::DlgTabBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
    myLabelList.setAutoDelete( TRUE );
}


DlgTabBar::~DlgTabBar()
{
}


void DlgTabBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new TabBarPage( this, parent, "TabBar" ) );

    labels.append( tr( "Tab Bar" ) );
}

void DlgTabBar::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "TabBar" );

  QTabBar* widget = GetTabBar();

  generator.WriteStartList( "TabNames" );
  QStrListIterator iter( myLabelList );
  const char* label;
  
  while( (label = iter()) )
    generator.WriteSingleLine( "Tab", label );

  generator.WriteEndBlock();

  generator.WriteSingleLine( "Shape", ShapeToStr( widget->shape() ) );
  
  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgTabBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QTabBar* widget = GetTabBar();

    // Compare to known keys
    if( key == "Tab" )
    {
	myLabelList.append( value );
    }
    else if( key == "Shape" )
    {
	widget->setShape( StrToShape( value ) );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgTabBar::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "TabBar" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetTabBar() )
    {
        CreateQtWidget( 0 );
    }
  
    QTabBar* widget = GetTabBar();

    XMLValueList list = map["TabNames"].AsList();
    QListIterator<XMLValue> it( list );
    for( ; it.current(); ++it )
        myLabelList.append( it.current()->AsQString() );

    widget->setShape( StrToShape( map["Shape"].AsQString() ) );

    // We inherit from 
    Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgTabBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QTabBar* widget = GetTabBar();

    stream << varName << "setShape( QTabBar::" << 
	ShapeToStr( widget->shape() ) << " );\n";

    stream << srcIndent << "{" << endl;
    bool first = TRUE;
    QStrListIterator iter( myLabelList );
    const char* label;
    while( (label = iter()) )
    {
	if( first )
	{
	    stream << srcIndent << srcIndent << "QTab* tab = 0;" << endl;
	    first = FALSE;
	}
	
	stream << srcIndent << srcIndent << "tab = new QTab;" << endl

               << srcIndent << srcIndent 
               << "tab->label = ";
        GenerateText( label, widgetLayout, stream );
        stream << ";" << endl

               << srcIndent << varName << "addTab( tab );" << endl

               << endl;
    }
    stream << srcIndent << "}" << endl;
}


void DlgTabBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgTabBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "selected(int)" );
    }

    return itsSignals;
}


QString DlgTabBar::ShapeToStr( QTabBar::Shape theShape )
{
    QString str;
    
    switch( theShape )
    {
	case QTabBar::RoundedAbove:
	{
	    str = "RoundedAbove";
	    break;
	}
	case QTabBar::RoundedBelow:
	{
	    str = "RoundedBelow";
	    break;
	}
	case QTabBar::TriangularAbove:
	{
	    str = "TriangularAbove";
	    break;
	}
	case QTabBar::TriangularBelow:
	{
	    str = "TriangularBelow";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return str;
}


QTabBar::Shape DlgTabBar::StrToShape( const QString& theShapeStr )
{
    QTabBar::Shape shape = QTabBar::RoundedAbove;
    
    if( theShapeStr == "RoundedAbove" )
    {
	shape = QTabBar::RoundedAbove;
    }
    else if( theShapeStr == "RoundedBelow" )
    {
	shape = QTabBar::RoundedBelow;
    }
    else if( theShapeStr == "TriangularAbove" )
    {
	shape = QTabBar::TriangularAbove;
    }
    else if( theShapeStr == "TriangularBelow" )
    {
	shape = QTabBar::TriangularBelow;
    }
    
    return shape;
}


void DlgTabBar::CreateQtWidget(
    const char*	theName
    )
{
    QTabBar* widget = 
	new QTabBar( GetParent(),
		     CreateName( "TabBar", 
				 theName, 
				 itsNextWidgetNum ) );

    QTab* tab = new QTab;
    tab->label = tr( "Tab 1" );
    widget->addTab( tab );

    tab = new QTab;
    tab->label = tr( "Tab 2" );
    widget->addTab( tab );

    tab = new QTab;
    tab->label = tr( "Tab 3" );
    widget->addTab( tab );

    SetQtWidget( widget );
}
