/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-1999 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgWidget.h,v 2.17 2000/05/09 02:01:42 jeff Exp $

*******************************************************************************/


#ifndef DlgWidget_included
#define DlgWidget_included

#include <qtstream.h>
#include <qwidget.h>
#include <qlist.h>
#include <qlistbox.h>
#include <qstring.h>
#include <qstrlist.h>

#include "widget/SignalConnection.h"

class QtArchGenerator;
class XMLValue;
class WidgetLayout;

class DlgWidget : public QObject
{
    friend class WidgetModuleMgr;

    Q_OBJECT
    
public:

    enum Type
    {
	DW_Button,
	DW_ButtonGroup,
	DW_CheckBox,
	DW_ComboBox,
	DW_Frame,
        DW_Grid,
	DW_GroupBox,
        DW_HBox,
	DW_HButtonGroup,
        DW_HGroupBox,
        DW_IconView,
	DW_LCDNumber,
	DW_Label,
	DW_LineEdit,
	DW_ListBox,
	DW_ListView,
	DW_MenuBar,
	DW_MultiLineEdit,
	DW_ProgressBar,
	DW_PushButton,
	DW_RadioButton,
	DW_ScrollBar,
	DW_ScrollView,
	DW_Slider,
	DW_SpinBox,
	DW_Splitter,
	DW_TabBar,
	DW_TextView,
	DW_TextBrowser,
	DW_User,
        DW_VBox,
	DW_VButtonGroup,
        DW_VGroupBox,
	DW_None = -1
    };
    
public:

    virtual ~DlgWidget();

    static DlgWidget *NewDlgWidget( 
	QWidget *, 
	DlgWidget::Type, 
	const QString * = NULL 
	);
    // Construct a new subclass using DlgWidget::Type

    static DlgWidget *NewDlgWidget( 
	QWidget *,
	const QString&,
	const QString* = NULL 
	);
    // Construct a new subclass using a Widgetname

    static DlgWidget *NewDlgWidget( 
	QWidget *,
  const XMLValue &
	);
    // Construct a new subclass using a XMLValue
    
    void SetParent( QWidget * );
    // Sets a parent, called from the NewDlgWidget and the module manager
    // methods 

    QWidget* GetWidget() const;
    // return the widget's widget

    QString GetWidgetName() const;
    // return the widget's name

    QPoint GetPos() const;
    // return the widget's position relative to its original parent

    Type GetType() const;
    // return the type of the widget


    //
    // Property Editing Methods
    //

    int DoPropertyDialog();
    // Do the property dialog

    virtual void GetPropertyPages(
	QWidget*		parent,
	QList<QWidget>&		pages,
	QStrList&		labels
	);
    // Get the property pages and their labels for the widget


    //
    // Saving and Loading Methods
    //

    virtual void Save( QtArchGenerator & ) = 0;
    // save the contents of the widget

    virtual bool RestoreKeyValue( 
	const QString&	key, 
	const QString&	value,
	bool		topLevel = TRUE
	);
    // Restore a single key/value pair

    virtual void RestoreXML( const XMLValue & ) = 0;
    // Restore from a given XMLValue

    //
    // Layout State Methods
    //

    bool IsInLayout() const;
    // Return true if the widget is in a layout or a layout menu bar

    void SetInLayout(
	QWidget*	newParent 
	);
    // Make the widget be in a layout.  Make it become a child of 
    // the given parent.  If the new parent is NULL, make it a child of its
    // original parent.

    void RemoveFromLayout();
    // remove the widget from a layout, and make it a child of its
    // original parent
    

    //
    // Code Generation Methods
    //

    void SetVariableMapping(
	const QString&	variableName
	);
    // if variableName != "", then the widget is mapped to a variable

    bool IsVariableMapped() const;
    // is the widget mapped to a variable

    const QString& GetVariableName() const;
    // return the name of the variable, or "" if not mapped

    QString GetGeneratedVarName() const;
    // return the name of the widget variable during generation, either the
    // name of the variable or a temporary name if not mapped

    virtual QString GetHeaderFilename() const = 0;
    // return the header filename for the widget
    
    virtual QString GetWidgetClass() const = 0;
    // return the class name for the widget

    const char* Dot() const;
    // return a '->' since all variables are allocated dynamically

    virtual void GenerateSource(
	QTextStream&		stream,
	const QString&		varName,
        const WidgetLayout&     widgetLayout
	);
    // Generate the source code needed to initialize the widget.
    // varName already has a leading tab and the correct dereferencer

    virtual void GenerateCtor(
	QTextStream&		stream,
	const QString&		parentName
	);
    // output a constructor for the widget.  The
    // ctor for leaves the output at the start of the next line

    void GenerateSourceConnects(
        QTextStream& stream, 
        const QString& varName
        );
    // Generate connection of signals to slots

    //
    // Connected Signals List
    //

    QList<SignalConnection>& GetConnectedSignalList();
    // return a reference to the list of signals

    void GetSignalList( 
	QStrList& signalList
	) const;
    // return a list of the names of the signals that can be connected
    // to this widget


    //
    // Background Pixmap methods
    //

    void SetBackgroundData( 
	bool useIt, 
	bool adjustSize, 
	bool addToProject, 
	bool readAsData, 
	const QString &varName, 
	const QString &lenName, 
	const QString &fileName 
	);
    // Setting data of the background pixmap

    void GetBackgroundData( 
	bool &useIt, 
	bool &adjustSize, 
	bool &addToProject, 
	bool &readAsData, 
	QString &varName, 
	QString &lenName, 
	QString &fileName 
	);
    // Setting data of the background pixmap

    bool UseBackgroundPixmap();
    // Returns true if a pixmap should be used for this widget

    const QPixmap& GetPixmap() const;
    // Get a pixmap for the widget.

    static const QPixmap& GetPixmap( DlgWidget::Type type );
    // Get a pixmap for the given widget type        

    DlgWidget * GetParentForCodeGeneration(
        const QList<DlgWidget>& zOrdering
        );
    // Get the widget whole will be this widgets parent when the code is
    // generated


    bool IsEnabled() const;
    
    void SetEnabled(
        bool    isEnabled
        );
    

    void ClearParentForSource();
    // Reset itsParentForSource  to NULL

    static QString srcIndent;
    // Text to use as the indentation when generating source

    //
    // Utility methods
    //

    static QString FocusPolicyToStr(
	QWidget::FocusPolicy	theFocusPolicy
	);
    // Return a string value for the focus policy

    static QWidget::FocusPolicy StrToFocusPolicy(
	const QString&	theFocusPolicyStr
	);
    // Return a focus policy enum for the string, defaulting to NoFocus

    static QString BackgroundModeToStr(
	QWidget::BackgroundMode	theBackgroundMode
	);
    // Return a string value for the background mode

    static QWidget::BackgroundMode StrToBackgroundMode(
	const QString&	theBackgroundModeStr
	);
    // Return a background mode enum for the string, defaulting to
    // palette background.

    static QString AlignmentToStr(
	int	theAlignment
	);
    // Return a string value for the given alignment

    static int StrToAlignment(
	const QString&	theAlignmentStr
	);
    // Return an alignment value for the given alignment string

    static QString BackgroundOriginToStr(
        QWidget::BackgroundOrigin       origin
        );
    static QWidget::BackgroundOrigin StrToBackgroundOrigin(
        const QString&  str
        );
    // BackgroundOrigin enum

    static QString TextFormatToStr(
        Qt::TextFormat  format
        );
    static Qt::TextFormat StrToTextFormat(
        const QString&  str
        );
    // Qt::TextFormat enum

signals: 

    void LayoutStatusChanged( 
	const DlgWidget* 
	);
    // The layout status of this widget has changed.

protected: //Methods

    DlgWidget(
	Type	type
	);
    // Constructor.

    virtual void CreateQtWidget(
	const char*	theName
	) = 0;
    // Create and return the correct Qt widget for the specific type of
    // DlgWidget.

    void SetQtWidget(
	QWidget*	theWidget
	);
    // Set the Qt widget.  This method is called by
    // CreateQtWidget.

    QWidget *GetParent();
    // Returns the parent 

    //
    // Connected Signals list
    //

    virtual void AddSignalsToList(
	QStrList&	signalList
	) const;
    // add signals specific to a particular widget type to the list


    //
    // Code Generation Methods
    //

    void GenerateText(    
        const QString&          text,
        const WidgetLayout&     widgetLayout,
        QTextStream&            stream,
        bool                    noTranslate = FALSE
        );
    // Generate the given text.  The text may be translated, depending on the
    // settings in the layout.  If not translated, the text will be quoted.


    //
    // Create a unique name for the widget
    //

    static QString CreateName( 
	QString, 
	const char *, 
	int & 
	);
    // Generates a unique WidgetName based on the Class name, a user-
    // provided name (no name generation) and a class specific number

    static void UpdateNextWidgetNumber( 
	const QString&	widgetName,
	int&		nextWidgetNum
	);


protected: //Attributes

    bool useBackgroundPixmap;
    // Use a background pixmap.

    static QString saveIndent;
    // Text to use as the indentation when saving the widget's properties.

private: // Methods

    QRect MapGeometryToOriginalParent( 
	const QWidget*	theWidget
	) const;
    // Map this widget's geometry to it's original parent.


private:

    QWidget*	itsWidget;
    // The Qt widget

    QString	itsVariableName;
    
    Type	itsType;

    QList<SignalConnection>	itsConnectedSignalList;

    bool	isInLayout;
    // Is the widget in a layout

    QWidget*	itsOriginalParent;
    // the original parent for this widget, the widget frame

    QString backgroundPixmapsName;
    // Filename of the background pixmap

    bool adjustSize;
    bool addPixmapToProject;
    bool readPixmapFromData;
    QString dataVarName;
    QString dataLenName;
    // Some switches for the background pixmap handling

    bool        isEnabled;
    // Is the widget enabled

    DlgWidget *itsParentForSource;  // Widget who will be this widget's parent when source is generated.
};


inline void DlgWidget::SetQtWidget(
    QWidget*	theWidget
    )
{
  itsWidget = theWidget;
}


inline QWidget* DlgWidget::GetWidget() const
{
    return itsWidget;
}


inline DlgWidget::Type DlgWidget::GetType() const
{
    return itsType;
}


inline QString DlgWidget::GetWidgetName() const
{
    return GetWidget()->name();
}

inline bool DlgWidget::IsInLayout() const
{
    return isInLayout;
}


inline bool DlgWidget::IsVariableMapped() const
{
    return !itsVariableName.isEmpty();
}


inline const QString& DlgWidget::GetVariableName() const
{
    return itsVariableName;
}


inline void DlgWidget::SetVariableMapping(
    const QString&	variableName
    )
{
    itsVariableName = variableName;
}

inline void DlgWidget::ClearParentForSource()
{
    itsParentForSource = NULL;
}

inline QList<SignalConnection>& DlgWidget::GetConnectedSignalList()
{
    return itsConnectedSignalList;
}

inline bool DlgWidget::UseBackgroundPixmap()
{
	return useBackgroundPixmap;
}

inline const QPixmap& DlgWidget::GetPixmap() const
{
    return GetPixmap( GetType() );
}

inline bool DlgWidget::IsEnabled() const
{
    return isEnabled;
}

inline void DlgWidget::SetEnabled(
    bool        isEnabled
    )
{
    this->isEnabled = isEnabled;
}

inline void DlgWidget::SetParent( QWidget *parent )
{
    itsOriginalParent = parent;
}

inline QWidget* DlgWidget::GetParent()
{
    return itsOriginalParent;
}


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QSize&	size 
    );

QTextStream& operator >>( 
    QTextStream&	istream,
    QSize&		size 
    );


QTextStream& operator <<( 
    QTextStream&	ostream,
    const QPoint&	point 
    );

QTextStream& operator >>( 
    QTextStream&	istream,
    QPoint&		point 
    );


QTextStream& operator <<(
    QTextStream&	ostream,
    const QRect&	rect
    );


QTextStream& operator >>(
    QTextStream&	istream,
    QRect&		rect
    );


// write palette to text stream
QTextStream& operator <<(
    QTextStream&	ostream,
    const QPalette&	palette
    );


// read palette from stream
QTextStream& operator >>(
    QTextStream&	istream,
    QPalette&		palette
    );


// write color group to text stream
QTextStream& operator <<(
    QTextStream&	ostream,
    const QColorGroup&	group
    );


// read color group from a text stream
QTextStream& operator >>(
    QTextStream&	istream,
    QColorGroup&	group
    );


// write font to text stream
QTextStream& operator <<(
    QTextStream&	ostream,
    const QFont&	font
    );


// read a font from the text stream
QTextStream& operator >>(
    QTextStream&	istream,
    QFont&		font
    );


void SaveBool(
    QTextStream&	ostream,
    const bool& 
    );

void RestoreBool(
    QTextStream&	istream,
    bool &
    );


#endif // DlgWidget_included


